#include "d3dbase.h"
#include "errorwindow.h"
#include "debugmsg.h"

cd3dbase::cd3dbase(HINSTANCE Instance, int WindowStyle, char *caption, int dx, int dy)
	:cwindow(Instance, WindowStyle, caption, dx, dy)
{
	cdebugmsg(DBG_HW, DBGCMD_CLEAR);
	cdebugmsg(DBG_HW, "cd3dbase::cd3dbase()...");
	fsxres = dx;
	fsyres = dy;
	this->WindowStyle = WindowStyle;
/*	if(WindowStyle == SW_SHOWMAXIMIZED)
		fullscreen = true;
	else*/
	if(WindowStyle == SW_SHOWMAXIMIZED)
	{
		xres = fsxres;
		yres = fsyres;
	}
	else
	{
		xres = clientdx;
		yres = clientdy;
	}
	fullscreen = false;
	cdebugmsg(DBG_HW, "cd3dbase::cd3dbase(): calling initd3d()");
	initd3d();
}

cd3dbase::onmessage(int msg, int wparam, int lparam)
{
	cwindow::onmessage(msg, wparam, lparam);
}

cd3dbase::onminimax(int WindowStyle)
{
	cdebugmsg(DBG_HW, "cd3dbase::onminimax()");
	bool oldfullscr = (this->WindowStyle == SW_SHOWMAXIMIZED);
	bool newfullscr = (WindowStyle == SW_SHOWMAXIMIZED);
	cdebugmsg(DBG_HW, oldfullscr?"old mode: fullscreen":"old mode: windowed");
	cdebugmsg(DBG_HW, newfullscr?"new mode: fullscreen":"new mode: windowed");
	cwindow::onminimax(WindowStyle);
	if(WindowStyle == SW_SHOWMAXIMIZED)
	{
		xres = fsxres;
		yres = fsyres;
	}
	else
	{
		xres = clientdx;
		yres = clientdy;
	}
	if(oldfullscr ^ newfullscr)
	{
		cdebugmsg(DBG_HW, "onminimax: calling reinitd3d...");
		reinitd3d();
	}
}

cd3dbase::onactivate()
{
	cdebugmsg(DBG_HW, "cd3dbase::onactivate()");
	cwindow::onactivate();
/*	if(fullscreen && WindowStyle == SW_SHOWMINIMIZED)
	{
		// restore fullscreen window
		ShowWindow(hwnd, SW_SHOWMAXIMIZED);
		fullscreen = false;
	}*/
}

cd3dbase::ondeactivate()
{
	cdebugmsg(DBG_HW, "cd3dbase::ondeactivate()");
	cwindow::ondeactivate();
	if(WindowStyle == SW_SHOWMAXIMIZED)
	{
		// remember that the window was fullscreen
		fullscreen = true;
		// render to window
		WindowStyle = SW_SHOWMINIMIZED;
		reinitd3d();
		// minimize
		ShowWindow(hwnd, SW_SHOWMINNOACTIVE);
	}
/*	if(WindowStyle == SW_SHOWMAXIMIZED)
	{
//		WindowStyle = SW_SHOWNORMAL;
//		reinitd3d();
		ShowWindow(hwnd, SW_SHOWNORMAL);
	}*/
}

cd3dbase::onsize(int dx, int dy)
{
	cdebugmsg(DBG_HW, "cd3dbase::onsize()...");
	cwindow::onsize(dx, dy);
	if(WindowStyle != SW_SHOWMAXIMIZED)
	{
		xres = dx;
		yres = dy;
	}
}

cd3dbase::reinitd3d()
{
/*	destroyd3d();
	initd3d();*/
	cdebugmsg(DBG_HW, "cd3dbase::reinitd3d()...");
	if(WindowStyle == SW_SHOWMAXIMIZED)
	{
		cdebugmsg(DBG_HW, "mode: fullscreen");
	  d3dpp.Windowed = FALSE;
	  d3dpp.BackBufferWidth   = fsxres;
	  d3dpp.BackBufferHeight  = fsyres;
	  d3dpp.BackBufferFormat  = D3DFMT_A8R8G8B8;
 	}
	else
	{
		cdebugmsg(DBG_HW, "mode: windowed");
		d3dpp.Windowed = TRUE;
		D3DDISPLAYMODE d3ddm;
		d3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);
		d3dpp.BackBufferFormat = d3ddm.Format;
	}
	d3dpp.AutoDepthStencilFormat = D3DFMT_D32;
	cdebugmsg(DBG_HW, "mode zbuffer: 32 bit");
	if(d3ddevice->Reset(&d3dpp) != D3D_OK)
	{
		cdebugmsg(DBG_HW, "failed");
		cdebugmsg(DBG_HW, "mode zbuffer: 16 bit");
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
		if(d3ddevice->Reset(&d3dpp) == D3D_OK)
			cdebugmsg(DBG_HW, "ok");
		else
			cdebugmsg(DBG_HW, "failed");
	}
}

cd3dbase::initd3d()
{
	cdebugmsg(DBG_HW, "cd3dbase::initd3d()...");
	d3d = NULL;
	d3ddevice = NULL;
	if(d3d = Direct3DCreate8(D3D_SDK_VERSION))
	{
		D3DDISPLAYMODE d3ddm;
		if(d3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm) == D3D_OK)
		{
			ZeroMemory( &d3dpp, sizeof(d3dpp) );
			d3dpp.BackBufferCount= 1;
			if(WindowStyle == SW_SHOWMAXIMIZED)
			{
				cdebugmsg(DBG_HW, "mode: fullscreen");
				d3dpp.Windowed = FALSE;
			}
			else
			{
				cdebugmsg(DBG_HW, "mode: windowed");
				d3dpp.Windowed = TRUE;
				d3dpp.BackBufferFormat = d3ddm.Format;
				d3dpp.BackBufferWidth = clientdx;
				d3dpp.BackBufferHeight = clientdy;
				d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
			}
			// enumerate display modes
			D3DDISPLAYMODE Enum, Mode, CompMode;
			Mode.Width = 0;
			CompMode.Width = 1000000;
			for(int i = 0; i < (int)d3d->GetAdapterModeCount(D3DADAPTER_DEFAULT); i++)
			{
				d3d->EnumAdapterModes(D3DADAPTER_DEFAULT, i, &Enum);
				if(((Enum.Width == 640 && Enum.Height == 480)
					|| (Enum.Width == fsxres && Enum.Height == fsyres))
					&& (Enum.Format == D3DFMT_R5G6B5 || Enum.Format == D3DFMT_A8R8G8B8 || Enum.Format == D3DFMT_X8R8G8B8))
				{
					if(Enum.Width > Mode.Width) Mode = Enum;
					if(Enum.Width < CompMode.Width) CompMode = Enum;
					if(Enum.Width == Mode.Width
						&& (Enum.Format == D3DFMT_A8R8G8B8 || Enum.Format == D3DFMT_X8R8G8B8) && (Mode.Format == D3DFMT_R5G6B5)) Mode = Enum;
					if(Enum.Width == CompMode.Width
						&& (Enum.Format == D3DFMT_R5G6B5) && (CompMode.Format == D3DFMT_A8R8G8B8 || CompMode.Format == D3DFMT_X8R8G8B8)) CompMode = Enum;
					if(Enum.Width == Mode.Width
						&& Enum.Format == Mode.Format
						&& Enum.RefreshRate > Mode.RefreshRate)
						Mode = Enum;
					if(Enum.Width == CompMode.Width
						&& Enum.Format == CompMode.Format
						&& Enum.RefreshRate < CompMode.RefreshRate)
						CompMode = Enum;
					char *mstr = (char *)malloc(1024);
					sprintf(mstr, "enum display mode (ok): %dx%d, %d bit, %d Hz", Enum.Width, Enum.Height, (Enum.Format == D3DFMT_R5G6B5)?16:32, Enum.RefreshRate);
					cdebugmsg(DBG_HW, mstr);
					delete mstr;
				}
				else
				{
					char *mstr = (char *)malloc(1024);
					sprintf(mstr, "enum display mode (rejected): %dx%d, %d, %d Hz", Enum.Width, Enum.Height, Enum.Format, Enum.RefreshRate);
					cdebugmsg(DBG_HW, mstr);
					delete mstr;
				}
			}
			char *mstr = (char *)malloc(1024);
			sprintf(mstr, "select display mode: %dx%d, %d bit, %d Hz", Mode.Width, Mode.Height, (Mode.Format == D3DFMT_R5G6B5)?16:32, Mode.RefreshRate);
			cdebugmsg(DBG_HW, mstr);
			delete mstr;
			if(Mode.Width == 0)
			{
				cdebugmsg(DBG_HW, "DirectX: Direct3D: Display Mode Enumeration returned no usable modes");
				cerrorwindow("DirectX: Direct3D: Display Mode Enumeration returned no usable modes", "D3D error");
				exit(0);
			}
			if(WindowStyle == SW_SHOWMAXIMIZED)
			{
				d3dpp.hDeviceWindow  = hwnd;
			}
			bool success = false;
			for(int dmode = 0; dmode < ((CompMode.Width!=1000000)?2:1) && !success; dmode++)
			{
				switch(dmode)
				{
				case 0:
					if(WindowStyle == SW_SHOWMAXIMIZED)
					{
						d3dpp.BackBufferWidth = Mode.Width;
						d3dpp.BackBufferHeight = Mode.Height;
						d3dpp.BackBufferFormat = Mode.Format;
						d3dpp.FullScreen_RefreshRateInHz = Mode.RefreshRate;
					}
					cdebugmsg(DBG_HW, "dmode: normal");
					break;
				case 1:
					if(WindowStyle == SW_SHOWMAXIMIZED)
					{
						d3dpp.BackBufferWidth = CompMode.Width;
						d3dpp.BackBufferHeight = CompMode.Height;
						d3dpp.BackBufferFormat = CompMode.Format;
						d3dpp.FullScreen_RefreshRateInHz = CompMode.RefreshRate;
					}
					cdebugmsg(DBG_HW, "dmode: compatibility");
					break;
				}
				for(int bits = 0; bits < 2 && !success; bits++)
				{
					switch(bits)
					{
					case 0:
						cdebugmsg(DBG_HW, "mode zbuffer: 32 bit");
						d3dpp.AutoDepthStencilFormat = D3DFMT_D32;
						break;
					case 1:
						cdebugmsg(DBG_HW, "mode zbuffer: 16 bit");
						d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
						break;
					}
					d3dpp.EnableAutoDepthStencil = true; // z buffer
					d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
					//d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
					d3dpp.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_DEFAULT;
					if(d3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
													D3DCREATE_SOFTWARE_VERTEXPROCESSING,
													&d3dpp, &d3ddevice) == D3D_OK)
					{
						cdebugmsg(DBG_HW, "CreateDevice successful");
						success = true;
					}
					else
					{
						cdebugmsg(DBG_HW, "CreateDevice failed");
					}
				}
			}
			if(!success)
			{
				cdebugmsg(DBG_HW, "DirectX: Direct3D: CreateDevice failed");
				cerrorwindow("DirectX: Direct3D: CreateDevice failed", "D3D error");
				exit(0);
			}
		}
		else
		{
			cerrorwindow("DirectX: Direct3D: GetAdapterDisplayMode failed", "D3D error");
			exit(0);
		}
	}
	else
	{
		cerrorwindow("DirectX: Direct3D: Direct3DCreate8 failed", "D3D error");
		exit(0);
	}
}

cd3dbase::destroyd3d()
{
	cdebugmsg(DBG_HW, "cd3dbase::destroyd3d()...");
	// Direct3D
	RELEASE(d3ddevice);
	RELEASE(d3d);
	cdebugmsg(DBG_HW, "ok");
}

cd3dbase::~cd3dbase()
{
	destroyd3d();
}

// d3dbase make functions

D3DVECTOR makevector(float x, float y, float z)
{
	D3DVECTOR v;
	v.x = x;
	v.y = y;
	v.z = z;
	return v;
};


D3DBVERTEX makevertex(D3DVECTOR w, D3DVECTOR n, float tu, float tv)
{
	D3DBVERTEX v;
	v.x = w.x;
	v.y = w.y;
	v.z = w.z;
	v.nx = n.x;
	v.ny = n.y;
	v.nz = n.z;
	v.tu = tu;
	v.tv = tv;
	return v;
};


D3DBLVERTEX makelvertex(D3DVECTOR w, D3DCOLOR c, float tu, float tv)
{
	D3DBLVERTEX v;
	v.x = w.x;
	v.y = w.y;
	v.z = w.z;
	v.color = c;
	v.tu = tu;
	v.tv = tv;
	return v;
};

D3DMATRIX makematrix(float a, float b, float c, float d, float e, float f, float g, float h, float i, float j, float k, float l, float m, float n, float o, float p)
{
	D3DMATRIX z;
	z._11 = a;
	z._12 = b;
	z._13 = c;
	z._14 = d;
	z._21 = e;
	z._22 = f;
	z._23 = g;
	z._24 = h;
	z._31 = i;
	z._32 = j;
	z._33 = k;
	z._34 = l;
	z._41 = m;
	z._42 = n;
	z._43 = o;
	z._44 = p;
	return z;
};