#ifndef FORMATS_H
#define FORMATS_H

#define STRINGLEN 80 // Standard length of strings
#define ORDERLINES 14

#define OMILS 400 // maximal number of vehicles
#define OMILTYPES 12 // Number of vehicle types 

// graphics
#define XRES 800//800
#define YRES 600//600
#define BITS 32// change DXRENDERDEPTH, too!
#define DXRENDERDEPTH DDBD_32

// debugging / profiling
//#define DEBUG // activate debug mode
//#define PROFILER // activate the profiler (bars on the screen)
//#define DISABLESOUND // disable sound
// disable shot lighting (shot lighting makes the game very slow,
// so it's always deactivated)
#define DISABLESHOTLIGHTING
//#define USERVIS // Show debug values as bars on the screen
#define USERVISNUM 7 // number of debug values
#define PROFILERNUM 15 // number of profiler parts

// zoom
#define MINZOOM 0.5f
#define MAXZOOM 1.5f
#define STDZOOM 1.3f
#define STDFPZOOM 1.0f
#define VIEWDIV 8 // some kind of zoom
#define MAXHEIGHT 6 // maximal height of the terrain
// Factors for terrain resolution change from Version 3.0 to 3.1
#define GEOFAC 1 // first try; had problems with this
#define TERRAFAC 2 // second try, works better
// Size of ground texture array
#define TEXX 8
#define TEXY 8
// ground texture resolution
#define TXRES 256
#define TYRES 256
// Num of texels per field,
// used for calculation of texture coordinates in terranorm()
#define FXRES ((float)(TXRES*TEXX)/(float)FIELDX)
#define FYRES ((float)(TYRES*TEXY)/(float)FIELDY)
#define TEXTSPACE2OFF 32// offset of text 2 textures
// OFFSET of the tree textures in the directx texture-array
#define TREESPACEOFF 33
// OFFSET of the text textures (used for the menu) in the directx texture-array
#define TEXTSPACEOFF 37
// OFFSET of the terrain textures in the directx texture-array
#define TEXSPACEOFF 41
// maximal number of  lines in the short mission description
#define ORDERLINES 14
#define TEXTOPACITY 180 // The opacity of the text screens (menu)
// "End of the world" constants
// Used in renderterra() for calculating what is visible
#define LEFTEND -24
#define RIGHTEND 24
#define TOPEND 27
#define BOTTOMEND -17
// This is what's added to topend in the intro to make more terrain visible
#define FIRSTTOPPLUS 32
// Number of missions displayed in the "Select mission" menu
#define DISPLAYEDMISSIONS 9 
#define TERRAWAYS 100 // Maximal number of ways that can be used in missions
// Maximal number of plains (under houses) that can be used in missions
#define TERRAPLAINS 100 
#define STRINGLEN 80 // Standard length of strings
#define MAXLIGHTS 5 // Maximum number of lights, not used anymore
// dx
#define STDCOMPFUNC D3DCMP_LESSEQUAL // Compare function of the z-buffer
#define ZBCLEARVALUE 1.0f // Clear value of the z-buffer
// Adjustment of the z-values to make objects be displayed correctly
#define ZMULT 0.4f
#define ZADD 0.9f
// Viewport settings for the z-buffer
#define VIEWPORTMINZ 0.0f
#define VIEWPORTMAXZ 1.0f
// Enable automatically (by directx) managed textures
#define TEXMANAGER true
// dsound
#define SOUNDS 5 // Number of different soundeffects
#define PARSOUNDS 40 // Maximal number of sounds played at the same time
// Sounds, which are only played once at a time
#define ONESOUNDS 1
// Minimum interval between two sound effects. This prevents sound chaos
// when many vehicles are shooting
#define SOUNDINTERVAL 0.025
// game
#define TREES 256 // number of trees
#define NUMSHOTS OMILS*20 // maximal number of shots (bullets)
// Number of 3d-objects (OMILTYPES*<you or enemy>*<tank or turret>)
#define OBJECTS OMILTYPES*4
#define PARTICLES 800 // max number of particles (explosions, smoke)
// Secondary particles in an explosion (set to 0 to make the game faster)
#define EXPPARTS 0
#define SHOTGRAVITY 2 // Gravitation effects
// particle effects
// particle lifetime
#define PARTICLELIFE 6 // explosions
#define PARTICLELIFE3 3 // ground hits
#define PARTICLELIFE4 3 // small explosions (hits)
#define PARTICLELIFE5 1.5f // driving smoke
// timing of explosion graphics
#define EXPLFIRE 0.25f // how long does the fire last
#define EXPLFULLSMOKEBEGIN 0.15f // smoke beginning to blend in
#define EXPLFULLSMOKEEND 0.4f// smoke beginning to blend out
#define EXPSIZE 4 // size of explosion particles
#define HITSIZE 1.0f // size of small explosion (hit) particles
//  < HITSIZE , used for randomly calculation of the particle size
#define HITSIZERANGE 0.8f
// the (relative) range where the artillery tanks start being more effective
#define MARSRANGE 0.6f
// font sizes
#define BIGX 16
#define BIGY 30
#define SMALLX 12
#define SMALLY 24
// gameplay
#define TANKSPEED 4
#define SHOTSPEED 8
#define TURNSPEED 4
#define TURRETTURNSPEED 2
#define TANKHITPOINTS0 100 // combat tank
#define TANKHITPOINTS1 60 // small tank
// when a tank is attacked, all other tanks within this distance, strike back
#define GROUPATTACKDST 16
// but only GROUPATTACKNUM of them attack the same vehicle
#define GROUPATTACKNUM 4
// debug
#ifdef DEBUG
// if DEBUG is defined, write the game status into a file,
// when status(long) is called
#define status(x) statusdebug((x))
#endif
#ifndef DEBUG
// if DEBUG is not defined, do something that doesn't need much time
// instead of writing the game status into a file
#define status(x) {int foo=0;}
#endif
#ifdef PROFILER
// PROFILER activated? use profile functions
#define startprofile(x) realstartprofile(x)
#define stopprofile(x) realstopprofile(x)
#endif
#ifndef PROFILER
// PROFILER deactivated? use dummy functions for faster execution
#define startprofile(x) {int foo=0;}
#define stopprofile(x) {int foo=0;}
#endif
// dx
// offset in directx's lighting structures for the particles
#define PARTLIGHTOFFSET 20
// offset in directx's lighting structures for the shots
#define SHOTLIGHTOFFSET 20+PARTICLES
// macros
// calculate normals for the lighting (math stuff)
#define DXXRESULT(a,b,c) -((a.y-b.y)*(c.z-b.z)-(a.z-b.z)*(c.y-b.y))
#define DXYRESULT(a,b,c) -((a.z-b.z)*(c.x-b.x)-(a.x-b.x)*(c.z-b.z))
#define DXZRESULT(a,b,c) -((a.x-b.x)*(c.y-b.y)-(a.y-b.y)*(c.x-b.x))
#define DXNORM(a,b,c) dxfix(DXXRESULT(a,b,c),DXYRESULT(a,b,c),DXZRESULT(a,b,c))

#endif