#include "matrixmath.h"

#include "t3_d3dbase.h"

// matrices for transforming (math stuff)
D3DMATRIX matrix, viewmtx, projmtx, vpmtx;


////////////////////// mapply2d precalculated //////////////////////
// For translations from World space to 2d screen space.
// Needed for selections by mouse and visibility check
void mapply2d(D3DVECTOR* vec)
{
  D3DVECTOR r;
  r = *vec;
  // apply
  vec->x=r.x*vpmtx.m[0][0]+r.y*vpmtx.m[1][0]+r.z*vpmtx.m[2][0]+vpmtx.m[3][0];
  vec->y=r.x*vpmtx.m[0][1]+r.y*vpmtx.m[1][1]+r.z*vpmtx.m[2][1]+vpmtx.m[3][1];
//  vec->z=r.x*vpmtx(0, 2)+r.y*vpmtx(1, 2)+r.z*vpmtx(2, 2)+vpmtx(3, 2);
  // scale
  float s = vpmtx.m[3][3]+r.x*vpmtx.m[0][3]+r.y*vpmtx.m[1][3]+r.z*vpmtx.m[2][3];
  if(s!=0.0f && s!=1.0f){
	s = 1/s;
    vec->x*=s;
    vec->y*=s;
//    vec->z*=s;
  }
}

////////////////////// mapply //////////////////////
// For translations from World space to screen space.
// Needed for selections by mouse and visibility check
void mapply(D3DVECTOR* v, D3DMATRIX mo){
  D3DMATRIX m=mo;
  D3DVECTOR r;
  // apply
  r.x=v->x*m.m[0][0]+v->y*m.m[1][0]+v->z*m.m[2][0];
  r.y=v->x*m.m[0][1]+v->y*m.m[1][1]+v->z*m.m[2][1];
  r.z=v->x*m.m[0][2]+v->y*m.m[1][2]+v->z*m.m[2][2];
  // move
  r.x+=m.m[3][0];
  r.y+=m.m[3][1];
  r.z+=m.m[3][2];
  // scale
  float s = m.m[3][3];
  s+=v->x*m.m[0][3];
  s+=v->y*m.m[1][3];
  s+=v->z*m.m[2][3];
  if(s!=0.0f){
	s = 1/s;
    r.x*=s;
    r.y*=s;
    r.z*=s;
  }
  *v=r;
}

////////////////////// mapply2d calculated //////////////////////
// For translations from World space to 2d screen space.
// Needed for selections by mouse and visibility check
void mapply2dreal(D3DVECTOR* vec)
{
  D3DVECTOR r;
  D3DMATRIX *v = &viewmtx;
  D3DMATRIX *p = &projmtx;
  // apply
  float s;
  r = *vec;
  // apply
  vec->x=r.x*(*v).m[0][0]+r.y*(*v).m[1][0]+r.z*(*v).m[2][0]+(*v).m[3][0];
  vec->y=r.x*(*v).m[0][1]+r.y*(*v).m[1][1]+r.z*(*v).m[2][1]+(*v).m[3][1];
  vec->z=r.x*(*v).m[0][2]+r.y*(*v).m[1][2]+r.z*(*v).m[2][2]+(*v).m[3][2];
  // scale
  s = (*v).m[3][3]+r.x*(*v).m[0][3]+r.y*(*v).m[1][3]+r.z*(*v).m[3][2];
  if(s!=0.0f && s!=1.0f){
	s = 1/s;
    vec->x*=s;
    vec->y*=s;
    vec->z*=s;
  }
  // apply
  r.x=vec->x*(*p).m[0][0]+vec->y*(*p).m[1][0]+vec->z*(*p).m[2][0]+(*p).m[3][0];
  r.y=vec->x*(*p).m[0][1]+vec->y*(*p).m[1][1]+vec->z*(*p).m[2][1]+(*p).m[3][1];
  // scale
  s = (*p).m[3][3]+vec->x*(*p).m[0][3]+vec->y*(*p).m[1][3]+vec->z*(*p).m[3][2];
  if(s!=0.0f && s!=1.0f){
	s = 1/s;
    r.x*=s;
    r.y*=s;
  }
  *vec=r;
}

////////////////////// mapply2d precalculated //////////////////////
// For translations from World space to 2d screen space.
// Needed for selections by mouse and visibility check
void mapply3d(D3DVECTOR* vec)
{
  D3DVECTOR r;
  r = *vec;
  // apply
  vec->x=r.x*vpmtx.m[0][0]+r.y*vpmtx.m[1][0]+r.z*vpmtx.m[2][0]+vpmtx.m[3][0];
  vec->y=r.x*vpmtx.m[0][1]+r.y*vpmtx.m[1][1]+r.z*vpmtx.m[2][1]+vpmtx.m[3][1];
  vec->z=r.x*vpmtx.m[0][2]+r.y*vpmtx.m[1][2]+r.z*vpmtx.m[2][2]+vpmtx.m[3][2];
  // scale
  float s = vpmtx.m[3][3]+r.x*vpmtx.m[0][3]+r.y*vpmtx.m[1][3]+r.z*vpmtx.m[2][3];
  if(s!=0.0f && s!=1.0f){
	s = 1/s;
    vec->x*=s;
    vec->y*=s;
    vec->z*=s;
  }
}
