#ifndef MISSION_H
#define MISSION_H

#include "constants.h"

////////////////////// vehicle description ///////////////////////
// this is used to store the vehicle's positions when a mission file (.mis)
// is loaded
//
// posx,posy:          the position of the tank
// targetx, targety:   the target position for movement
// a:                  angle
// typ:                type, same as in "struct mil"
// target:             the tanks target
// the bool constants: same as in "struct mil"
// owner:              0:you, 1:enemy, 2:civil but hostile, 3:civil and neutral
struct tomildesc {
  float posx, posy, targetx, targety, a;
  int typ, target;
  bool noorders, nogoal, mustsurvive, mustdie, thisisstatic;
  unsigned char owner;
};

/////////////////////// light settings //////////////////////////
// Light as it is defined in the mission description file (.mis)
// Two directional light sources can be defined in a .mis-file
// The light rays are parallel
//
// x,y,z: direction vector of the light
// r,g,b: color of the light splitted in red, green and blue
struct tmissionlight {
  float x,y,z;
  float r,g,b;
};

////////////////////// mission goal ////////////////////////////
// survive:      index for a unit that must survive
//               not used anymore, must be set to -1
// kill:         index for a unit that must die
//               not used anymore, must be set to -1
// killall:      "kill all enemies"
// location:     "move to the following location".
//               The location is defined by gotox,gotoy and radius
// killlocation: All enemies in a specific area must be killed.
//               The area is defined by killx,killy,killradius
// ban:          No enemy is allowed to move to the defined area
//               The area is defined by banx,bany,banradius
struct tgoal {
  int survive, kill;
  bool killall, location, killlocation, ban;
  float gotox, gotoy, radius;
  float killx, killy, killradius;
  float banx, bany, banradius;
};

///////////////////// mission data ///////////////////////
// name:     Name of the mission
// fname:    Filename of the mission file (.mis, .int for the intro)
// next:     File name of the next mission
// orders:   The short orders of the mission
// numomil:  Number of vehicles in the mission
// namelen:  Length of the mission name string
// orderlen: Length of the mission short order strings
// nextlen:  Length of the next mission filename
// goal:     The goal(s) of the mission (see tgoal)
// omildesc: Descriptions of vehicles (see tomildsc)
// success:  Mission is done successfully
// done:     Mission is done
// snow:     Terrain has snow textures
// light:    The light settings of the mission (see tmissionlight)
struct tmission {
  char name[STRINGLEN];
  char fname[STRINGLEN];
  char next[STRINGLEN];
  char orders[ORDERLINES][STRINGLEN];
  int numomil, namelen, orderlen[ORDERLINES], nextlen;
  tgoal goal;
  tomildesc* omildesc;
  bool success, done, snow;
  tmissionlight light[2];
};

extern tmission mission;

void loadmission(char *fname);
void loadgame(char *fname);
void savegame(char *fname);

#endif
