#ifndef OMIL_H
#define OMIL_H

#include "constants.h"

////////////// description of a vehicle or house //////////////////
// this is used to store capabilities and constants of the vehicle type
//
// canshoot: false for standard jeeps, buses, lorries, civil cars and houses
struct tmildsc {
  float maxspeed, shotspeed, size, range, shothit, hitpoints, shotfreq;
  bool canshoot;
};

///////////////////// vehicle or house ///////////////////////
// vehicles and houses are stored in the same data structure
//
// x,y,z:           position
// a,b,c:           angles
// ta,tb:           turret angles
// da:              change value of a per frame
// sp:              current speed of the vehicle
// tdx,tdy,tdy:     vector of the direction where the turret points
//                  this is used to precalculate the shot movement
// tdp:             turret displacement
//                  used to let the turret move back, when it shoots
// sx,sy:           relative screen coordinates
//                  used to calculate the selection of units with the mouse
// targetx,targety: ki target coordinates (for movement)
// owner:           0: you, 1: enemy, 2: civil but hostile, 3:civil and neutral
// target:          the target of the tank
// ttarget:         the target of the turret
// mtarget:         true, if the target is defined by the mission description
//                  if this is true, the target will not be changed by the ai
//                  automatically
// hit:             current hitpoints of the tank
// maxhit:          the maximal hitpoints of the tank
// typ:             type of the tank
//                  0:  Leopard II (combat tank, german army)
//                  1:  Marder (small tank, german army)
//                  2:  MARS (artillery tank, german army)
//                  3:  hummvee (Jeep, US Army)
//                  4:  pritsche (multiporpose unarmed vehicle, german army)
//                  5:  tonner (lorry, german army)
//                  6:  bus (bus, german army)
//                  7:  wolf (multipurpose unarmed jeep, german army)
//                  8:  house of stone (civil)
//                  9:  house of wood (civil)
//                  10: golf (civil car, red)
//                  11: golf (civil car, blue)
// lastfoot:        time index of last "footprint" on the terrain
// lastshot:        time index of last shot
// lastsmoke:       time index of last wheel smoke
// used:            used or empty
// selected:        currently selected and awaiting new orders
// hitting:         for ai, it's set to false of the tanks shot doesn't hit its
//                  target. When this is false the ai makes the tank drive
//                  towards the target to hit better
// dead:            This unit is shot. But it's still rendered with the
//                  "dead.bmp" texture and acts like a static object in
//                  the game.
// noorders:        the user can't give orders to this vehicle. This is used
//                  for guarding and convoy missions
// nogoal:          when the mission goal is "kill all enemies" and an enemies
//                  "nogoal" flag is set, you don't have to kill this enemy.
//                  nicht zur Strke (german insider joke)
// mustsurvive:     This unit must survive
// mustdie:         This unit must die
// thisisstatic:    This doesn't move. Used for houses, dead tanks, civil cars
//
// programmers comment:
// "Hope this was the worst part of the documentation..."
struct mil {
  float x, y, z, a, b, c, tb, sp, da, tdx, tdy, tdz, ta, tdp;
  float sx, sy;
  float targetx, targety;
  char owner;
  int target, ttarget;
  bool mtarget;
  float hit, maxhit;
  int typ;
  float lastfoot, lastshot, lastsmoke;
  bool used, selected, hitting, dead, noorders, nogoal, mustsurvive, mustdie;
  bool thisisstatic;
};

void rendermil(mil* milobj);
void shoot(int n);
void mildscinit(void);
void attack(int n, int tgt);
void ki(int n);
void calcomil();

extern tmildsc mildsc[OMILTYPES];

extern mil omil[OMILS];

#endif