#include "overlay.h"

#include "gs.h"
#include "t3_d3dbase.h"
#include "tank31fct.h"
#include "mission.h"

#include <stdio.h>

// font arrays, loaded from "fontbig.bmp" and "fontsmal.bmp"
unsigned char fontbig[BIGX*BIGY*256], fontsmall[SMALLX*SMALLY*256];

//////////////// load font //////////////////////
// loads a font from a bitmap
void t3loadfont(unsigned char* font, int width, int height, char* fname){
  int i, j;
  FILE *f;
  f=fopen(fname, "rb");
  fseek(f, -width*height*256*3, SEEK_END);
  unsigned int r, g, b;
  for(j=0;j<height;j++){
    for(i=0;i<256*width;i++){
      fread(&r, 1, 1, f);
      fread(&g, 1, 1, f);
      fread(&b, 1, 1, f);
      font[((height-1)-j)*256*width+i]=r;
    }
  }
  fclose(f);
}

void newmenu(int n){
  view.newmenu=true;
  view.newmenun=n;
}

void displaymenu(int n){
  view.menuscroll=0;
  view.menumaxscroll=0;
  if(view.menutype!=n&&!view.firstperson)loadmenu(n);
  view.menutype=n;
  view.menu=true;
}

////////////// textout ////////////////////
// displays text on a texture (used for the menu)
void T3TextOut(bool big, unsigned char* font,int textur, int xt, int y, char* str, int len){
  int i, j, k;
  D3DLOCKED_RECT lrect;
  status(1700);
  if(dxstexture[textur]->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
  status(1701);
  unsigned int *texbuf=(unsigned int*)lrect.pBits;
  unsigned short *texbufw=(unsigned short*)lrect.pBits;
  status(1702);
  if(lrect.pBits==NULL){statusreal(707);dxerror();}
  status(1703);
  unsigned int fchr;
    for(k=0;k<len;k++){
      int x=xt+(big?BIGX:SMALLX)*k;
      for(j=0;j<(big?BIGY:SMALLY);j++){
        if(y+j>=0&&y+j<256){
          for(i=0;i<(big?BIGX:SMALLX);i++){
            if(x+i>=0&&x+i<256){
              fchr=font[256*(big?BIGX:SMALLX)*j+str[k]*(big?BIGX:SMALLX)+i];
              texbuf[256*(y+j)+(x+i)]=(TEXTOPACITY<<24)+(fchr<<16)+(fchr<<8)+fchr;
            }
          }
        }
      }
    }
  status(1705);
  if(dxstexture[textur]->UnlockRect(0)!=D3D_OK)dxerror();
  status(1706);
}

//////////// cred textout /////////////////
// the same as t3textout, but limited to a scrolling area
void T3CredTextOut(bool big, unsigned char* font,int textur, int xt, int y, char* str, int len, int yn){
  int i, j, k;
  D3DLOCKED_RECT lrect;
  status(1700);
  if(dxstexture[textur]->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
  status(1701);
  unsigned int *texbuf=(unsigned int*)lrect.pBits;
  unsigned short *texbufw=(unsigned short*)lrect.pBits;
  status(1702);
  if(lrect.pBits==NULL){statusreal(707);dxerror();}
  status(1703);
  unsigned int fchr;
    for(k=0;k<len;k++){
      int x=xt+(big?BIGX:SMALLX)*k;
      for(j=0;j<(big?BIGY:SMALLY);j++){
        if((y+j>=60||(yn==1&&y+j>=0))&&(y+j<144||(yn==0&&y+j<=256))){
          for(i=0;i<(big?BIGX:SMALLX);i++){
            if(x+i>=0&&x+i<256){
              fchr=font[256*(big?BIGX:SMALLX)*j+str[k]*(big?BIGX:SMALLX)+i];
              texbuf[256*(y+j)+(x+i)]=(TEXTOPACITY<<24)+(fchr<<16)+(fchr<<8)+fchr;
            }
          }
        }
      }
    }
  status(1705);
  if(dxstexture[textur]->UnlockRect(0)!=D3D_OK)dxerror();
  status(1706);
}

/////////////// text out 4 ////////////////
// Used to draw the text parted into the four menu textures
// The menu is divided into 4 textures to let texture resolution stay <=256
void T3TextOut4(bool big, unsigned char* font,int textur, int xt, int y, char* str, int len){
  T3TextOut(big, font, textur, xt*2, y*2, str, len);
  T3TextOut(big, font, textur+1, xt*2-256, y*2, str, len);
  T3TextOut(big, font, textur+2, xt*2, y*2-256, str, len);
  T3TextOut(big, font, textur+3, xt*2-256, y*2-256, str, len);
}

////////////// cred text out 4 //////////////
// same, but limited to the scrolling area
void T3CredTextOut4(bool big, unsigned char* font,int textur, int xt, int y, char* str, int len){
  T3CredTextOut(big, font, textur, xt*2, y*2, str, len, 0);
  T3CredTextOut(big, font, textur+1, xt*2-256, y*2, str, len, 0);
  T3CredTextOut(big, font, textur+2, xt*2, y*2-256, str, len, 1);
  T3CredTextOut(big, font, textur+3, xt*2-256, y*2-256, str, len, 1);
}

////////////////////// display credits ////////////////////
// Display scrolling credit text
void displaycredits(void){
  view.pixscroll+=4*speedgs.c;
  if(view.pixscroll>=19*12*2-10)view.pixscroll=0;
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+30-(int)view.pixscroll, "Project manager:", 16);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+42-(int)view.pixscroll, "  Christian Reimann", 19);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+54-(int)view.pixscroll, "Programmer:", 11);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+66-(int)view.pixscroll, "  Christian Reimann", 19);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+78-(int)view.pixscroll, "Beta testers:", 13);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+90-(int)view.pixscroll, "  Susanne Sass", 14);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+102-(int)view.pixscroll, "  Daniel Nielsen", 16);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+114-(int)view.pixscroll, "  Gerhard \"skyreacher\" Dietrichsteiner", 38);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+126-(int)view.pixscroll, "3D Model designers:", 19);
//  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+138-(int)view.pixscroll, "  Helge \"Hiding Dragon\" Backhaus", 32);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+138-(int)view.pixscroll, "  Christian Reimann", 19);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+150-(int)view.pixscroll, "Code optimizers:", 16);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+162-(int)view.pixscroll, "  Christian Reimann", 19);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+174-(int)view.pixscroll, "  Daniel Nielsen", 16);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+186-(int)view.pixscroll, "Mission designers:", 18);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+198-(int)view.pixscroll, "  Christian Reimann", 19);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+210-(int)view.pixscroll, "  Travis Sharp", 14);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+222-(int)view.pixscroll, "Sound effects:", 14);
  T3CredTextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 170+234-(int)view.pixscroll, "  Christian Reimann", 19);
}

////////////// load menu //////////////////////
// load a menu type and draw the text
void loadmenu(int n){
  int i, j, k;
  D3DLOCKED_RECT lrect;
  status(1630);
  status(1633);
  for(k=0;k<4;k++){
    if(dxstexture[TEXTSPACEOFF+k]->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
    unsigned int *texbuf=(unsigned int*)lrect.pBits;
    unsigned short *texbufw=(unsigned short*)lrect.pBits;
    if(lrect.pBits==NULL){statusreal(708);dxerror();}
    for(i=0;i<256;i++){
      for(j=0;j<256;j++){
          if((i<=1&&(k%2)==0)||(i>=254&&(k%2)==1)||(j<=1&&k<=1)||(j>=254&&k>=2)){
            texbuf[256*j+i]=(TEXTOPACITY<<24)+0xFFFFFF;
          } else{
            texbuf[256*j+i]=(TEXTOPACITY<<24);
          }
      }
    }
    if(dxstexture[TEXTSPACEOFF+k]->UnlockRect(0)!=D3D_OK)dxerror();
  }
  status(1641);
  status(1602);
  status(1603);
  if(n==0){
    status(1632);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 10, mission.name, mission.namelen);
    status(1633);
    for(i=0;i<ORDERLINES;i++)if(mission.orders[i]!=NULL)T3TextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 40+i*12, mission.orders[i], mission.orderlen[i]);
    T3TextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 220, "press 'O' to show / hide orders", 31);
  }
  status(1620);
/*
0:	Orders
1:	Game Menu
2:	Mission accomplished/failed
3:	Select Single mission
4:	Credits
5:	last mission accomplished
6:	Main Menu
7:	load game
8:	save game
*/
  if(n==1){
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 10, "Game Menu", 9);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 50, "Orders", 6);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 70, "Start New Game", 14);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 90, "Load Game", 9);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 110, "Save Game", 9);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 130, "Start Single Mission", 20);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 150, "Credits", 7);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 170, "Fullscreen / Windowed", 21);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 190, "Quit", 4);
  }
  if(n==6){
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 10, "Main Menu", 9);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 50, "Start New Game", 14);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 70, "Load Game", 9);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 90, "Start Single Mission", 20);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 110, "Credits", 7);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 130, "Fullscreen / Windowed", 21);
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 150, "Quit", 4);
  }
  status(1621);
  if(n==4){
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 10, "Credits", 7);
    displaycredits();
    T3TextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 210, "Find updates, new versions and", 30);
    T3TextOut4(false, &fontsmall[0], TEXTSPACEOFF, 10, 220, "support on www.infinitecode.com", 31);
  }
  status(1622);
  if(n==2){
    if( mission.success)T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 20, 100, "Mission accomplished", 20);
    if(!mission.success)T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 50, 100, "Mission failed", 14);
  }
  status(1623);
  if(n==3){
    for(i=0;i<DISPLAYEDMISSIONS;i++){
      strcpy(view.mfnames[i], "");
    }
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 10, "New Mission", 11);
    WIN32_FIND_DATA data;
    int num=0;
    HANDLE h=FindFirstFile("missions\\*.mis", &data);
    if(view.menuscroll==0){
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 50, data.cFileName, strlen(data.cFileName));
      strcpy(view.mfnames[0], data.cFileName);
    }
    while(FindNextFile(h, &data)){
      num++;
      if(num-view.menuscroll>=0&&num-view.menuscroll<DISPLAYEDMISSIONS){
        T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 50+20*(num-view.menuscroll), data.cFileName, strlen(data.cFileName));
        strcpy(view.mfnames[num-view.menuscroll], data.cFileName);
      }
    }
    view.menumaxscroll=num-(DISPLAYEDMISSIONS-1);
    FindClose(h);
  }
  if(n==7){
    for(i=0;i<DISPLAYEDMISSIONS;i++){
      strcpy(view.mfnames[i], "");
    }
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 10, "Load Game", 9);
    WIN32_FIND_DATA data;
    int num=0;
    HANDLE h=FindFirstFile("save\\*.t3s", &data);
    if(view.menuscroll==0){
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 50, data.cFileName, strlen(data.cFileName));
      strcpy(view.mfnames[0], data.cFileName);
    }
    while(FindNextFile(h, &data)){
      num++;
      if(num-view.menuscroll>=0&&num-view.menuscroll<DISPLAYEDMISSIONS){
        T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 50+20*(num-view.menuscroll), data.cFileName, strlen(data.cFileName));
        strcpy(view.mfnames[num-view.menuscroll], data.cFileName);
      }
    }
    view.menumaxscroll=num-(DISPLAYEDMISSIONS-1);
    FindClose(h);
  }
  if(n==8){
    for(i=0;i<DISPLAYEDMISSIONS;i++){
      strcpy(view.mfnames[i], "");
    }
    T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 10, "Save Game", 9);
    WIN32_FIND_DATA data;
    int num=0;
    HANDLE h=FindFirstFile("save\\*.t3s", &data);
    if(view.menuscroll==0){
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 50, data.cFileName, strlen(data.cFileName));
      strcpy(view.mfnames[0], data.cFileName);
    }
    while(FindNextFile(h, &data)){
      num++;
      if(num-view.menuscroll>=0&&num-view.menuscroll<DISPLAYEDMISSIONS){
        T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 50+20*(num-view.menuscroll), data.cFileName, strlen(data.cFileName));
        strcpy(view.mfnames[num-view.menuscroll], data.cFileName);
      }
    }
    view.menumaxscroll=num-(DISPLAYEDMISSIONS-1);
    FindClose(h);
  }
  if(n==5){
    if(mission.success){
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 20, 60, "Mission accomplished", 20);
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 20, 80, "Now check", 9);
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 20, 100, "www.infinitecode.com", 20);
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 20, 120, "for updates of Tank 3", 23);
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 20, 140, "and new stuff!", 14);
    } else {
      T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 50, 100, "Mission failed", 14);
    }
  }
  status(1604);
  T3TextOut4(true, &fontbig[0], TEXTSPACEOFF, 10, 235, "press ESC to toggle menu", 24);
  status(1605);
  status(1606);
  status(1607);
  status(1608);
  if(dxstexture[TEXTSPACEOFF+1]->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
  status(1609);
  unsigned int* texbuf=(unsigned int*)lrect.pBits;
  unsigned short* texbufw=(unsigned short*)lrect.pBits;
  status(1610);
  if(lrect.pBits==NULL){statusreal(709);dxerror();}
  status(1611);
  if(n==3){
      for(i=0;i<10;i++){
        texbuf[(80+10+i)*256*2+245*2]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80+10+i)*256*2+245*2+1]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80-10-i)*256*2+245*2]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80-10-i)*256*2+245*2+1]=(TEXTOPACITY<<24)+0xFFFFFF;
      }
      for(i=0;i<5;i++){
        texbuf[(80+20-i)*256*2+(245-i)*2]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80+20-i)*256*2+(245-i)*2+1]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80+20-i)*256*2+(245+i)*2]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80+20-i)*256*2+(245+i)*2+1]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80-20+i)*256*2+(245-i)*2]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80-20+i)*256*2+(245-i)*2+1]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80-20+i)*256*2+(245+i)*2]=(TEXTOPACITY<<24)+0xFFFFFF;
        texbuf[(80-20+i)*256*2+(245+i)*2+1]=(TEXTOPACITY<<24)+0xFFFFFF;
      }
  }
  if(dxstexture[TEXTSPACEOFF+1]->UnlockRect(0)!=D3D_OK)dxerror();
  status(1614);
}

///////////// init text space 2 /////////////////
// init a second text texture for the frames per second-display and
// the mouse cursor
void inittextspace2(){
  int i, j;
  D3DLOCKED_RECT lrect;
  if(dxstexture[TEXTSPACE2OFF]->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
  unsigned int *texbuf=(unsigned int*)lrect.pBits;
  unsigned short *texbufw=(unsigned short*)lrect.pBits;
  if(lrect.pBits==NULL){statusreal(710);dxerror();}
  for(i=0;i<256;i++){
    for(j=0;j<256;j++){
        texbuf[256*j+i]=(TEXTOPACITY<<24);
    }
  }
  for(i=0;i<13;i++){
    for(j=25;j<39;j++){
        texbuf[256*(j)+i]=0;
    }
  }
  for(i=0;i<=10;i++){
      texbuf[256*(25+i)]=(TEXTOPACITY<<24)+0xFFFFFF;
      texbuf[256*(25+i)+i/2]=(TEXTOPACITY<<24)+0xFFFFFF;
      texbuf[256*(25+10)+i/2]=(TEXTOPACITY<<24)+0xFFFFFF;
  }
  if(dxstexture[TEXTSPACE2OFF]->UnlockRect(0)!=D3D_OK)dxerror();
}

//////////// reinit text spasce 2 /////////////
// reinit the text space to show the new frames per second number
void reinittextspace2(){
  status(2100);
  if(speedgs.lc!=0){
    status(2110);
    float fps=1/speedgs.lc;
    status(2111);
    int ifps=(int)fps;
    char str[STRINGLEN];
    int num=0;
    while(ifps>0){
      num++;
      ifps/=10;
    }
    ifps=(int)fps;
    if(num<STRINGLEN-8){
      for(int i=0;i<num;i++){
        str[num-1-i]=48+(ifps%10);
        ifps/=10;
      }
      str[num]=' ';
      str[num+1]='f';
      str[num+2]='p';
      str[num+3]='s';
      str[num+4]=' ';
      str[num+5]=' ';
      str[num+6]=' ';
      str[num+7]=' ';
      str[num+8]=0;
      T3TextOut(false, &fontsmall[0], TEXTSPACE2OFF, 0, 0, str, strlen(str));
    }
    status(2112);
  }
  status(2113);
}
