#include "t3_d3dbase.h"
#include "gs.h"
#include "tank31fct.h"
#include "debugmsg.h"

ct3_d3dbase::onmessage(int msg, int wparam, int lparam)
{
	POINT p;
	GetCursorPos(&p);
	if(WindowStyle == SW_SHOWMAXIMIZED)
	{
		mouse.x=p.x;
		mouse.y=p.y;
	}
	else
	{
		RECT rect;
		GetClientRect(hwnd, &rect);
		POINT a;
		a.x = rect.left;
		a.y = rect.top;
		ClientToScreen(hwnd, &a);
		mouse.x=p.x-a.x;
		mouse.y=p.y-a.y;
		if(mouse.x < 0) mouse.x = 0;
		if(mouse.x > xres) mouse.x = xres;
		if(mouse.y < 0) mouse.y = 0;
		if(mouse.y > yres) mouse.y = yres;
	}
}

ct3_d3dbase::onmousemove(int x, int y, int buttons)
{
	cd3dbase::onmousemove(x, y, buttons);
	if(initcomplete)
	{
		if(((buttons&MK_LBUTTON)==MK_LBUTTON)&&!mouse.down){
			mouse.down=true;
			if(WindowStyle == SW_SHOWMAXIMIZED)
			{
				float fx=((2.0f*mouse.x)/XRES-1.0f),fy=(1.0f-(2.0f*mouse.y)/YRES);
				mouse.downx=fx;
				mouse.downy=fy;
			}
			else
			{
				RECT rect;
				GetClientRect(hwnd, &rect);
				POINT a, b;
				a.x = rect.left;
				a.y = rect.top;
				ClientToScreen(hwnd, &a);
				b.x = rect.right;
				b.y = rect.bottom;
				ClientToScreen(hwnd, &b);
				float fx=       ((float)(mouse.x-a.x)/(b.x-a.x))*2.0f-1.0f;
				float fy=1.0f - ((float)(mouse.y-a.y)/(b.y-a.y))*2.0f;
				mouse.downx=fx;
				mouse.downy=fy;
			}
		}
		if(((buttons&MK_RBUTTON)==MK_RBUTTON)&&!mouse.rdown){
			mouse.rdown=true;
		}
		if(((buttons&MK_LBUTTON)!=MK_LBUTTON)&&mouse.down&&!mouse.up){
			mouse.up=true;
		}
	}
}

ct3_d3dbase::onrbuttondown(int x, int y)
{
	cd3dbase::onrbuttondown(x, y);
	if(initcomplete)
	{
		mouse.rdown=true;
	}
}

ct3_d3dbase::onlbuttondown(int x, int y)
{
	cd3dbase::onlbuttondown(x, y);
	if(initcomplete)
	{
		if(!mouse.down){
			if(WindowStyle == SW_SHOWMAXIMIZED)
			{
				float fx=((2.0f*mouse.x)/XRES-1.0f),fy=(1.0f-(2.0f*mouse.y)/YRES);
				mouse.downx=fx;
				mouse.downy=fy;
			}
			else
			{
				RECT rect;
				GetClientRect(hwnd, &rect);
				POINT a, b;
				a.x = rect.left;
				a.y = rect.top;
				ClientToScreen(hwnd, &a);
				b.x = rect.right;
				b.y = rect.bottom;
				ClientToScreen(hwnd, &b);
				float fx=       ((float)(mouse.x)/(b.x-a.x))*2.0f-1.0f;
				float fy=1.0f - ((float)(mouse.y)/(b.y-a.y))*2.0f;
				mouse.downx=fx;
				mouse.downy=fy;
			}
			mouse.down=true;
		}
	}
}

ct3_d3dbase::onlbuttonup(int x, int y)
{
	cd3dbase::onrbuttonup(x, y);
	if(initcomplete)
	{
		mouse.up=true;
	}
}

ct3_d3dbase::ontimer()
{
	POINT p;
	GetCursorPos(&p);
	if(WindowStyle == SW_SHOWMAXIMIZED)
	{
		mouse.x=p.x;
		mouse.y=p.y;
	}
	else
	{
		RECT rect;
		GetClientRect(hwnd, &rect);
		POINT a;
		a.x = rect.left;
		a.y = rect.top;
		ClientToScreen(hwnd, &a);
		mouse.x=p.x-a.x;
		mouse.y=p.y-a.y;
		if(mouse.x < 0) mouse.x = 0;
		if(mouse.x > xres) mouse.x = xres;
		if(mouse.y < 0) mouse.y = 0;
		if(mouse.y > yres) mouse.y = yres;
	}
}

ct3_d3dbase::ct3_d3dbase(HINSTANCE Instance, int WindowStyle, char *caption, int dx, int dy)
:cd3dbase(Instance, WindowStyle, caption, dx, dy)
{
	SetTimer(hwnd, 42, 100, NULL);
	ShowCursor(false);
}

ct3_d3dbase::~ct3_d3dbase()
{
	KillTimer(hwnd, 42);
	ShowCursor(true);
}

ct3_d3dbase::onsize(int dx, int dy)
{
	cdebugmsg(DBG_HW, "ct3_d3dbase::onsize()...");
	//reinitd3d();
}

ct3_d3dbase::reinitd3d()
{
	cd3dbase::reinitd3d();
    gs->d3d->d3ddevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
    gs->d3d->d3ddevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    gs->d3d->d3ddevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR);
	gs->d3d->d3ddevice->SetRenderState(D3DRS_ZENABLE,TRUE);
	gs->d3d->d3ddevice->SetRenderState(D3DRS_ZWRITEENABLE,TRUE);
	gs->d3d->d3ddevice->SetRenderState(D3DRS_ZFUNC,STDCOMPFUNC);
	gs->d3d->d3ddevice->SetRenderState(D3DRS_LIGHTING,TRUE); 
	gs->d3d->d3ddevice->SetRenderState( D3DRS_AMBIENT, 0x20202020 );
	gs->d3d->d3ddevice->SetRenderState(D3DRS_SPECULARENABLE,FALSE); 
}