#include "gs.h"
//#include "xFile.h"
#include "tank31fct.h"
#include "terra.h"
#include "render.h"
#include "mission.h"
#include "omil.h"
#include "overlay.h"
#include "input.h"
#include "sound.h"
#include "matrixmath.h"
#include "debugmsg.h"

#include <d3d8.h>
#include <dsound.h>

// includes
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "t3math.h"
#include <time.h>
#include <sys/timeb.h>
#include <dos.h>


// resources for game structs


// game structs

tspeed speedgs;

tshot shot[NUMSHOTS];

ttree tree[TREES];

tparticle part[PARTICLES];

dx3dobject obj[OBJECTS];

tmouse mouse;

tview view;

///////////////////// mission data ///////////////////////
// name:     Name of the mission
// fname:    Filename of the mission file (.mis, .int for the intro)
// next:     File name of the next mission
// orders:   The short orders of the mission
// numomil:  Number of vehicles in the mission
// namelen:  Length of the mission name string
// orderlen: Length of the mission short order strings
// nextlen:  Length of the next mission filename
// goal:     The goal(s) of the mission (see tgoal)
// omildesc: Descriptions of vehicles (see tomildsc)
// success:  Mission is done successfully
// done:     Mission is done
// snow:     Terrain has snow textures
// light:    The light settings of the mission (see tmissionlight)

///////////////////// statistics /////////////////////////
// Used for debugging
//
// uservis:  Values to be displayed on the screen as bars (for debugging)
// profiler: speed of the game parts, for making the game faster
// pstart:   start time indices for the profiler
// ptime:    time consumed by different parts of the game (profiler)
struct tstats {
  float uservis[USERVISNUM];
  float profiler[PROFILERNUM];
  float pstart[PROFILERNUM];
  float ptime[PROFILERNUM];
} stats;

// gloabal variables

// windows
HWND hwnd; // window handle
MSG messages;// window messages, like "keypressed"
WNDCLASS wincl;// window class
BOOL bQuit=FALSE;// quit notification
char szClassName[STRINGLEN];// classname of the window class
// directx
D3DVIEWPORT8 dxviewport;// viewport (for setting coordinates and the z-buffer)
// Direcg3D GUIDs, hardware descriptions for selecting the direct3d device
GUID dxhwguid, dxrgbguid;
// used to store the graphic card's capabilities
bool dxhwfound, dxrgbfound, dxhwddfound, dxrgbddfound, dxsoftware;
// light description
D3DLIGHT8 dxlight;
// material description
D3DMATERIAL8 dxmat;
// texturing
// texture handle
IDirect3DTexture8 *dxstexture[TEXSPACEOFF+TEXX*TEXY];
// another handle
IDirect3DTexture8 *dxtexture;
// number of bits in the z-buffer
unsigned long dxzbits=0;
// foobar for dummy functions
int foo;
// true, if the game was started completely, for window events
bool initcomplete=false;
//XMESH mesh;

// function prototypes
LRESULT CALLBACK WindowProcedure2(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);


//////////// procedures + functions

////////////////////// fixed length for normals //////////////
// this function calculates vectors with a fixed length.
// used for normals
D3DVECTOR dxfix(float x, float y, float z){
  float invlen=1/(float)sqrt(x*x+y*y+z*z);
  D3DVECTOR v=makevector(x*invlen, y*invlen, z*invlen);
  return v;
}

//////////////////////// debug message ////////////////////////
// this writes a debug number into the file "status.txt"
// used to force debugging message
void statusreal(long s){
  FILE *f;
  f=fopen("status.txt", "w");
  fseek(f, 0, SEEK_SET);
  fwrite( &s, sizeof(s), 1, f);
  fclose(f);
}

//////////////////////// debug message ////////////////////////
// this writes a float number into the file "status.txt" rather than a long
void statusfloat(float s){
  FILE *f;
  f=fopen("status.txt", "w");
  fseek(f, 0, SEEK_SET);
  fwrite( &s, sizeof(s), 1, f);
  fclose(f);
}

//////////////////////// debug message ////////////////////////
// this writes a debug number into the file "status.txt".
// only used, if DEBUG is defined
void statusdebug(long s){
  FILE *f;
  f=fopen("status.txt", "w");
  fseek(f, 0, SEEK_SET);
  fwrite( &s, sizeof(s), 1, f);
  fclose(f);
}

//////////////////// profiling /////////////////////////
// take the start time of a game part for the profiler
void realstartprofile(int n){
  struct timeb t2;
  ftime(&t2);
  stats.pstart[n]=(float)clock()/CLOCKS_PER_SEC;
}

//////////////////// profiling /////////////////////////
// take the stop time of a game part for the profiler
// and update ptime
void realstopprofile(int n){
  struct timeb t2;
  ftime(&t2);
//  stats.ptime[n]=(249.0*stats.ptime[n]+(0.001*(1000*t2.time+t2.millitm)-stats.pstart[n]))/250.0;
  stats.ptime[n]+=((float)clock()/CLOCKS_PER_SEC-stats.pstart[n]);
}

///////////////////////// dxcleanup /////////////////////
// release all direct3d and directdraw structures
void dxcleanup(void){
	RELEASE(terra.vbterra);
	RELEASE(terra.vbdetail);
  for(int i=0;i<TEXSPACEOFF+TEXX*TEXY;i++)RELEASE(dxstexture[i]);
  if (gs->d3d->d3ddevice){
//    gs->d3d->d3ddevice->DeleteViewport(gs->d3dviewport);
  }
}

///////////////////////// dsoundshutdown /////////////////////
// release all directsound structures
void dsoundshutdown(void){
  int i;
  for(i=0;i<SOUNDS;i++){
    if(dxsoundbuffer[i]!=NULL)dxsoundbuffer[i]->Release();
  }
  for(i=0;i<ONESOUNDS;i++){
    if(dxonesoundbuffer[i]!=NULL)dxonesoundbuffer[i]->Release();
  }
  if(dxsound!=NULL)dxsound->Release();
}

////////////////////////// dxerror ////////////////////////
// Used to cleanly release directx and windows objects and to exit
void dxerror(void){
  dxcleanup();
  dsoundshutdown();
//  ShowCursor(true);
//  DestroyWindow(hwnd);
  exit(0);
}

////////////////////////// memerror ////////////////////////
// Used to cleanly release directx and windows objects and to exit
void memerror(void){
  dxcleanup();
  dsoundshutdown();
//  ShowCursor(true);
//  DestroyWindow(hwnd);
  exit(0);
}

////////////////////////// winerror ////////////////////////
// Used to cleanly release windows objects and to exit
void winerror(void){
//  ShowCursor(true);
//  DestroyWindow(hwnd);
  exit(0);
}

/////////////// load a 3d-model /////////////////
// A 3d-model is loaded from a .3do file
void dxloadobject(dx3dobject* obj, char* fname){
  int i, j;
  status(800);
  FILE *f;
  unsigned short typ;
  float scalex=1, scaley=1, scalez=1, fl=0;
  D3DVECTOR v[3];
  float tu[3], tv[3];
  status(801);
  obj->lists=obj->strips=obj->fans=0;
  obj->list=NULL;
  obj->strip=NULL;
  obj->fan=NULL;
  status(802);
  f=fopen(fname, "rb");
  fseek(f, 0, SEEK_END);
  int size=ftell(f);
  fseek(f, 0, SEEK_SET);
  status(803);
  while(/*!feof(f)*/ftell(f)<size){
    status(804);
    fread(&typ, 2, 1, f);
    status(805);
    if(typ==0){
      fread(&scalex, 4, 1, f);
      fread(&scaley, 4, 1, f);
      fread(&scalez, 4, 1, f);
    }
    status(806);
    if(typ==1){
      status(820);
      obj->lists++;
      if(obj->list==NULL){
        obj->list=(dxtriangle*)malloc(obj->lists*sizeof(dxtriangle));
        if(obj->list==NULL)memerror();
      } else {
        obj->list=(dxtriangle*)realloc(obj->list, obj->lists*sizeof(dxtriangle));
        if(obj->list==NULL)memerror();
      }
      status(821);
      fread(&fl, 4, 1, f);obj->list[obj->lists-1].num=((long)fl)/3;
      fread(&fl, 4, 1, f);obj->list[obj->lists-1].tex=(unsigned)fl;
	  status(822);
      obj->list[obj->lists-1].v=(D3DBVERTEX*)malloc(obj->list[obj->lists-1].num*3*sizeof(D3DBVERTEX));
      if(obj->list[obj->lists-1].v==NULL)memerror();
      status(823);
      for(i=0;i<obj->list[obj->lists-1].num/* *3/3 */;i++){
        status(824);
        for(j=0;j<3;j++){
          fread(&fl, 4, 1, f);v[j].x=scalex*fl;
          fread(&fl, 4, 1, f);v[j].y=scaley*fl;
          fread(&fl, 4, 1, f);v[j].z=scalez*fl;
          fread(&tu[j], 4, 1, f);
          fread(&tv[j], 4, 1, f);
        }
        status(825);
        for(j=0;j<3;j++){
          obj->list[obj->lists-1].v[i*3+j]=makevertex(v[j], DXNORM(v[0], v[1], v[2]),tu[j], tv[j]);
        }  
        status(826);
      }
    }
    status(807);
    if(typ==2){
      obj->strips++;
      if(obj->strip==NULL){
        obj->strip=(dxtriangle*)malloc(obj->strips*sizeof(dxtriangle));
        if(obj->strip==NULL)memerror();
      } else {
        obj->strip=(dxtriangle*)realloc(obj->strip, obj->strips*sizeof(dxtriangle));
        if(obj->strip==NULL)memerror();
      }
      fread(&fl, 4, 1, f);obj->strip[obj->strips-1].num=(long)fl-2;
      fread(&fl, 4, 1, f);obj->strip[obj->strips-1].tex=(unsigned)fl;
      obj->strip[obj->strips-1].v=(D3DBVERTEX*)malloc((obj->strip[obj->strips-1].num+2)*sizeof(D3DBVERTEX));
      if(obj->strip[obj->strips-1].v==NULL)memerror();
      // first 3
      for(i=0;i<3;i++){
        fread(&fl, 4, 1, f);v[i].x=scalex*fl;
        fread(&fl, 4, 1, f);v[i].y=scaley*fl;
        fread(&fl, 4, 1, f);v[i].z=scalez*fl;
        fread(&tu[i], 4, 1, f);
        fread(&tv[i], 4, 1, f);
      }
      for(i=0;i<3;i++){
        obj->strip[obj->strips-1].v[i]=makevertex(v[i], DXNORM(v[0], v[1], v[2]),tu[i], tv[i]);
      }  
      if(obj->strip[obj->strips-1].num+2>3){
        for(i=3;i<obj->strip[obj->strips-1].num+2;i++){
          fread(&fl, 4, 1, f);v[(i-3)%3].x=scalex*fl;
          fread(&fl, 4, 1, f);v[(i-3)%3].y=scaley*fl;
          fread(&fl, 4, 1, f);v[(i-3)%3].z=scalez*fl;
          fread(&tu[(i-3)%3], 4, 1, f);
          fread(&tv[(i-3)%3], 4, 1, f);
          if(i%2==0){
            obj->strip[obj->strips-1].v[i]=makevertex(v[(i-3)%3], DXNORM(v[0], v[1], v[2]),tu[(i-3)%3], tv[(i-3)%3]);
          } else {
            obj->strip[obj->strips-1].v[i]=makevertex(v[(i-3)%3], DXNORM(v[1], v[0], v[2]),tu[(i-3)%3], tv[(i-3)%3]);
          }
        }
      }
    }
    status(808);
  }
  status(809);
  fclose(f);
  status(810);
}

/////////////////// hwstatus ///////////////////
// write debug messages to hardware.txt
void hwstatus(char* str){
  FILE* f;
  f=fopen("hardware.txt", "a");
  fwrite(str,1,strlen(str),f);
  fclose(f);
}

void mipmapcreate(IDirect3DTexture8 *tex, int width, int height)
{
	int i, j, k;
	D3DLOCKED_RECT lrect;
  unsigned int* secmap=(unsigned int*)malloc(256*256*4);
  if(tex->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
  status(607);
  unsigned int *texbuf=(unsigned int*)lrect.pBits;
  status(608);
  if(lrect.pBits==NULL){statusreal(706);dxerror();}
  status(609);
  for(i=0;i<height;i++){
    for(j=0;j<width;j++){
         secmap[((height-1)-i)*256+j]=texbuf[((height-1)-i)*width+j];
    }
  }    
	IDirect3DSurface8 *nextlevel;
    int res=width;
  for(i = 1; i < (int)tex->GetLevelCount(); i++)
  {
      res/=2;
	  if(tex->GetSurfaceLevel(i, &nextlevel) == D3D_OK)
	  {
		  if(nextlevel->LockRect(&lrect, NULL, 0)!=D3D_OK)dxerror();
		  unsigned int* texbuf=(unsigned int*)lrect.pBits;
		  if(lrect.pBits==NULL){statusreal(709);dxerror();}
		  for(j=0;j<128;j++){
			for(k=0;k<128;k++){
			  secmap[j*256+k]=((((secmap[2*j*256+2*k]>>16)&0xFF)
							   +((secmap[(2*j+1)*256+2*k]>>16)&0xFF)
							   +((secmap[2*j*256+2*k+1]>>16)&0xFF)
							   +((secmap[(2*j+1)*256+2*k+1]>>16)&0xFF))/4)*0x010000
							 +((((secmap[2*j*256+2*k]>>8)&0xFF)
							   +((secmap[(2*j+1)*256+2*k]>>8)&0xFF)
							   +((secmap[2*j*256+2*k+1]>>8)&0xFF)
							   +((secmap[(2*j+1)*256+2*k+1]>>8)&0xFF))/4)*0x0100
							 +(((secmap[2*j*256+2*k]&0xFF)
							   +(secmap[(2*j+1)*256+2*k]&0xFF)
							   +(secmap[2*j*256+2*k+1]&0xFF)
							   +(secmap[(2*j+1)*256+2*k+1]&0xFF))/4)*0x01
							 +((((secmap[2*j*256+2*k]>>24)&0xFF)
							   +((secmap[(2*j+1)*256+2*k]>>24)&0xFF)
							   +((secmap[2*j*256+2*k+1]>>24)&0xFF)
							   +((secmap[(2*j+1)*256+2*k+1]>>24)&0xFF))/4)*0x01000000;
			}
		  }
		  for(j=0;j<res;j++){
			for(k=0;k<res;k++){
				texbuf[j*res+k]=secmap[j*256+k];
			}
		  }
		  if(nextlevel->UnlockRect()!=D3D_OK)dxerror();
	  }
    }
  free(secmap);
}

///////////////////////// load texture //////////////////////////
// load a texture from a file
//
// to:    the texture handle number
// fname: filename of the texture bitmap (.bmp)
// alphamap: automatically generate alpha values based on the colors
// mipmap:   when set to false, the function generates less mipmap levels
void loadtex(int to, char* fname, int width, int height, bool alphamap, bool mipmap){
  int i, j;
  D3DLOCKED_RECT lrect;
  status(600);
  FILE *bm;
  unsigned char *buf;
  buf=(unsigned char*)VirtualAlloc(NULL, width*height*3, MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE);
  if(buf==NULL)memerror();
  bm=fopen(fname, "rb");
  fseek(bm, -width*height*3, SEEK_END);
  fread(buf, 1, width*height*3, bm);
  fclose(bm);
  status(601);
  gs->d3d->d3ddevice->CreateTexture(width, height, mipmap?6:3, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[to]);
  status(604);
  status(605);
  status(606);
  if(dxstexture[to]->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
  status(607);
  unsigned int *texbuf=(unsigned int*)lrect.pBits;
  status(608);
  if(lrect.pBits==NULL){statusreal(706);dxerror();}
  status(609);
  for(i=0;i<height;i++){
    for(j=0;j<width;j++){
        int alpha=buf[(i*width+j)*3]+buf[(i*width+j)*3+1]+buf[(i*width+j)*3+2];
        if(alphamap){
          if(alpha<128){
            alpha=2*alpha;
          }else{
            alpha=255;
          }
        } else {
          alpha=0xFF;
        }
        texbuf[((height-1)-i)*width+j]=(alpha<<24)+(buf[(i*width+j)*3+2]<<16)+(buf[(i*width+j)*3+1]<<8)+buf[(i*width+j)*3];
    }
  }    
  status(610);
  if(dxstexture[to]->UnlockRect(0)!=D3D_OK)dxerror();
  status(611);
  mipmapcreate(dxstexture[to], width, height);
  status(613);
  VirtualFree(buf, width*height*3, MEM_DECOMMIT|MEM_RELEASE);
  status(614);
}

///////////////////// init text textures ///////////////////
void inittexttex(void){
  status(1500);
  gs->d3d->d3ddevice->CreateTexture(256, 256, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[TEXTSPACEOFF]);
  gs->d3d->d3ddevice->CreateTexture(256, 256, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[TEXTSPACEOFF]+1);
  gs->d3d->d3ddevice->CreateTexture(256, 256, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[TEXTSPACEOFF]+2);
  gs->d3d->d3ddevice->CreateTexture(256, 256, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[TEXTSPACEOFF]+3);
  gs->d3d->d3ddevice->CreateTexture(256, 256, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[TEXTSPACE2OFF]);
  status(1510);
  inittextspace2();
  loadmenu(0);
  status(1511);
}

///////////// init textures ///////////////
// generate and load textures
void dxinittex(void){
  int i, j;
  D3DLOCKED_RECT lrect;
  status(400);
  gs->d3d->d3ddevice->CreateTexture(32, 32, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[1]);
  gs->d3d->d3ddevice->CreateTexture(256, 256, 6, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[2]);
  if(dxstexture[2]->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
  unsigned int* texbuf=(unsigned int*)lrect.pBits;
  unsigned short* texbufw=(unsigned short*)lrect.pBits;
  if(lrect.pBits==NULL){statusreal(709);dxerror();}
  unsigned int* secmap=(unsigned int*)malloc(256*256*4);
  int width=256;
  int height=256;
  FILE *bm;
  unsigned char *buf;
  buf=(unsigned char*)VirtualAlloc(NULL, width*height*3, MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE);
  if(buf==NULL)memerror();
  bm=fopen("textures\\detail.bmp", "rb");
  fseek(bm, -width*height*3, SEEK_END);
  fread(buf, 1, width*height*3, bm);
  fclose(bm);
  unsigned char alpha=128;
  for(i=0;i<height;i++){
    for(j=0;j<width;j++){
        texbuf[((height-1)-i)*width+j]=(alpha<<24)+(buf[(i*width+j)*3+2]<<16)+(buf[(i*width+j)*3+1]<<8)+buf[(i*width+j)*3];
        secmap[((height-1)-i)*width+j]=texbuf[((height-1)-i)*width+j];
    }
  }
  VirtualFree(buf, width*height*3, MEM_DECOMMIT|MEM_RELEASE);
  for(i=0;i<256;i++){
      texbuf[i*256+255]=texbuf[i*256+0];
  }
  for(i=0;i<128;i++){
      texbuf[255*256+i]=texbuf[0*256+i];
  }
  if(dxstexture[2]->UnlockRect(0)!=D3D_OK)dxerror();
  mipmapcreate(dxstexture[2], width, height);
  statusreal(2500);
  statusreal(2501);
  gs->d3d->d3ddevice->CreateTexture(256, 256, 6, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &dxstexture[8]);
  if(dxstexture[8]->LockRect( 0, &lrect, NULL, 0 )!=D3D_OK)dxerror();
  texbuf=(unsigned int*)lrect.pBits;
  texbufw=(unsigned short*)lrect.pBits;
  if(lrect.pBits==NULL){statusreal(709);dxerror();}
  width=256;
  height=256;
  buf=(unsigned char*)VirtualAlloc(NULL, width*height*3, MEM_RESERVE|MEM_COMMIT, PAGE_READWRITE);
  if(buf==NULL)memerror();
  bm=fopen("textures\\details.bmp", "rb");
  fseek(bm, -width*height*3, SEEK_END);
  fread(buf, 1, width*height*3, bm);
  fclose(bm);
  alpha=128;
  for(i=0;i<height;i++){
    for(j=0;j<width;j++){
        texbuf[((height-1)-i)*width+j]=(alpha<<24)+(buf[(i*width+j)*3+2]<<16)+(buf[(i*width+j)*3+1]<<8)+buf[(i*width+j)*3];
        secmap[((height-1)-i)*width+j]=texbuf[((height-1)-i)*width+j];
    }
  }
  VirtualFree(buf, width*height*3, MEM_DECOMMIT|MEM_RELEASE);
  for(i=0;i<256;i++){
      texbuf[i*256+255]=texbuf[i*256+0];
  }
  for(i=0;i<128;i++){
      texbuf[255*256+i]=texbuf[0*256+i];
  }
  if(dxstexture[8]->UnlockRect(0)!=D3D_OK)dxerror();
  mipmapcreate(dxstexture[8], width, height);
  statusreal(2500);
  statusreal(2501);
  if(secmap!=NULL)free(secmap);
  statusreal(2503);
  // tex.explosion
  loadtex(0, "textures\\cam1.bmp", 64, 64, false, true);
  status(408);
//  loadtex(2, "textures\\detail.bmp", 64, 64, true, true);
  loadtex(3, "textures\\expl.bmp", 256, 256, false, true);
  // tex.rauch
  loadtex(4, "textures\\rauch.bmp", 256, 256, false, true);
  loadtex(5, "textures\\cam2.bmp", 64, 64, false, true);
  loadtex(6, "textures\\art.bmp", 64, 64, false, true);
  loadtex(7, "textures\\windows1.bmp", 64, 64, false, true);
//  loadtex(8, "textures\\details.bmp", 256, 256, true, true);
  loadtex(9, "textures\\windows2.bmp", 64, 64, false, true);
  loadtex(10, "textures\\dead.bmp", 256, 256, true, false);
  loadtex(11, "textures\\wolf.bmp", 128, 128, false, true);
  loadtex(12, "textures\\wolf2.bmp", 128, 128, false, true);
  loadtex(13, "textures\\house.bmp", 128, 128, false, true);
  loadtex(14, "textures\\golf.bmp", 128, 128, false, true);
  loadtex(15, "textures\\golf2.bmp", 128, 128, false, true);
  loadtex(16, "textures\\tank.bmp", 256, 256, false, true);
  loadtex(17, "textures\\leo.bmp", 256, 256, false, true);
  // TEXTSPACE2OFF=32
  status(409);
  // tex.tree
  loadtex(TREESPACEOFF, "textures\\tree.bmp", 256, 256, true, false);
  // tex.bush
  loadtex(TREESPACEOFF+1, "textures\\bush.bmp", 256, 256, true, false);
  loadtex(TREESPACEOFF+2, "textures\\tree2.bmp", 256, 256, true, false);
  loadtex(TREESPACEOFF+3, "textures\\tree3.bmp", 256, 256, true, false);
  status(410);
  // tex.terra
  initterratex();
  // tex.text zuletzt damit, wenn Videospeicher knapp ist, in Systemspeicher
  status(411);
}

/////////////// set material opacity //////////////////
// set the light reflection settings
void setmatopacity(float diffa, float diff, float amb, float spec){
  // mat.objects
//  RELEASE(dxmat);
//  gs->d3d->CreateMaterial(&gs->d3dmat, NULL);
  memset(&dxmat, 0, sizeof(D3DMATERIAL8));
  dxmat.Diffuse.r = diff;
  dxmat.Diffuse.g = diff;
  dxmat.Diffuse.b = diff;
  dxmat.Diffuse.a = diffa;
  dxmat.Ambient.r = amb;
  dxmat.Ambient.g = amb;
  dxmat.Ambient.b = amb;
  dxmat.Ambient.a = diffa;
  dxmat.Specular.r = spec;// Licht spiegeln
  dxmat.Specular.g = spec;
  dxmat.Specular.b = spec;
  dxmat.Specular.a = spec;
  dxmat.Power = 0.0f;
//  dxmat.power = (float)10.0;
  if(gs->d3d->d3ddevice->SetMaterial(&dxmat)!=D3D_OK)dxerror();
}

///////////////// init light //////////////////
// init the directional lights
void dxinitlight(){
  // material
//  gs->d3d->CreateMaterial(&gs->d3dmat, NULL);
  memset(&dxmat, 0, sizeof(D3DMATERIAL8));
  dxmat.Diffuse.r = 1.0;
  dxmat.Diffuse.g = (D3DVALUE)1.0;
  dxmat.Diffuse.b = (D3DVALUE)1.0;
  dxmat.Diffuse.a = (D3DVALUE)1.0;
  dxmat.Ambient.r = 1.0;
  dxmat.Ambient.g = (D3DVALUE)1.0;
  dxmat.Ambient.b = (D3DVALUE)1.0;
  dxmat.Ambient.a = (D3DVALUE)1.0;
  dxmat.Specular.r = (D3DVALUE)0.0;// Licht spiegeln
  dxmat.Specular.g = (D3DVALUE)0.0;
  dxmat.Specular.b = (D3DVALUE)0.0;
  dxmat.Power = 0.0f;
  // mat.background
  if(gs->d3d->d3ddevice->SetMaterial(&dxmat)!=D3D_OK)dxerror();
//  gs->d3dmat->GetHandle(gs->d3d->d3ddevice, &dxhmat);
//  gs->d3dviewport->SetBackground(dxhmat);
  // mat.objects
  setmatopacity(1, 1, 1, 1);
  // lighting
//  gs->d3d->CreateLight(&gs->d3dlight, NULL);
  memset(&dxlight, 0, sizeof(D3DLIGHT8));
  dxlight.Type = D3DLIGHT_DIRECTIONAL;
  dxlight.Diffuse.r = mission.light[0].r;
  dxlight.Diffuse.g = mission.light[0].g;
  dxlight.Diffuse.b = mission.light[0].b;
  dxlight.Specular.r=dxlight.Diffuse.r;
  dxlight.Specular.g=dxlight.Diffuse.g;
  dxlight.Specular.b=dxlight.Diffuse.b;
  dxlight.Attenuation0 = 0.0;
  dxlight.Attenuation1 = 0.0;
  dxlight.Attenuation2 = 0.0;
  dxlight.Range = sqrtf(FLT_MAX);
  dxlight.Position.x = 0;
  dxlight.Position.y = 0;
  dxlight.Position.z = 0;
  dxlight.Direction.x = mission.light[0].x;
  dxlight.Direction.y = mission.light[0].y;
  dxlight.Direction.z = mission.light[0].z;
  if(gs->d3d->d3ddevice->SetLight(1, &dxlight)!=D3D_OK)dxerror();
  if(gs->d3d->d3ddevice->LightEnable(1, true)!=D3D_OK)dxerror();
//  gs->d3dviewport->AddLight(gs->d3dlight);
//  gs->d3d->CreateLight(&gs->d3dlight, NULL);
  dxlight.Diffuse.r = mission.light[1].r;
  dxlight.Diffuse.g = mission.light[1].g;
  dxlight.Diffuse.b = 0.2f*0+mission.light[1].b;
  dxlight.Specular.r=dxlight.Diffuse.r;
  dxlight.Specular.g=dxlight.Diffuse.g;
  dxlight.Specular.b=dxlight.Diffuse.b;
  dxlight.Direction.x = mission.light[1].x;
  dxlight.Direction.y = mission.light[1].y;
  dxlight.Direction.z = mission.light[1].z;
  if(gs->d3d->d3ddevice->SetLight(2, &dxlight)!=D3D_OK)dxerror();
  if(gs->d3d->d3ddevice->LightEnable(2, true)!=D3D_OK)dxerror();
//  gs->d3dviewport->AddLight(gs->d3dlight);
}

bool dxinitzbuf(GUID dxguid, bool videomem){
  return true;
}


void releaseprimary(void){
}

void dxinit(void){
	cdebugmsg(DBG_HW, "dxinit()...");
  status(1110);
  status(1111);
  // Init DirectX and Direct3D
  // create direct draw object on primary surface
  status(1112);
  dxhwddfound=false;
  dxrgbddfound=false;
  dxhwfound=false;
  dxrgbfound=false;
  // init hardware.txt
  FILE *f;
  f=fopen("hardware.txt", "w");
  fclose(f);
  status(300);

  hwstatus("Init textures...\n");
  // textures
  status(320);
  dxinittex();
  status(321);
  // state
  hwstatus("Init RenderState...\n");
  if(!dxsoftware){
    gs->d3d->d3ddevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
    gs->d3d->d3ddevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    gs->d3d->d3ddevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR);
  }
/*  gs->d3d->d3ddevice->SetTextureStageState(0, D3DTSS_MIPMAPLODBIAS, 10000.0);
  gs->d3d->d3ddevice->SetRenderState(D3DRS_MIPMAPLODBIAS,10000.0);*/
  gs->d3d->d3ddevice->SetRenderState(D3DRS_ZENABLE,TRUE);//D3DZB_USEW
  gs->d3d->d3ddevice->SetRenderState(D3DRS_ZWRITEENABLE,TRUE);
  gs->d3d->d3ddevice->SetRenderState(D3DRS_ZFUNC,STDCOMPFUNC);
  gs->d3d->d3ddevice->SetRenderState(D3DRS_LIGHTING,TRUE);
  gs->d3d->d3ddevice->SetRenderState( D3DRS_AMBIENT, 0x40404040 );
//  gs->d3d->d3ddevice->SetRenderState( D3DRS_AMBIENT, 0xFFFFFFFF );
  gs->d3d->d3ddevice->SetRenderState(D3DRS_SPECULARENABLE,FALSE);
//  gs->d3d->d3ddevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CCW);
//  gs->d3d->d3ddevice->SetRenderState(D3DRS_EDGEANTIALIAS,TRUE);
//  gs->d3d->d3ddevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
  matrix=makematrix(1,0,0,0, 0,1,0,0, 0,0,1,0.5, 0,0,0,1);
  gs->d3d->d3ddevice->SetTransform(D3DTS_PROJECTION, &matrix);
  matrix=makematrix(1,0,0,0, 0,1,0,0, 0,0,ZMULT,0, 0,0,ZADD,1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_PROJECTION, &matrix);
  gs->d3d->d3ddevice->GetTransform(D3DTS_PROJECTION, &projmtx);
  // load objects
  hwstatus("Loading objects...\n");
  status(322);
  dxloadobject(&obj[0], "models\\tank.3do");
  dxloadobject(&obj[1], "models\\turret.3do");
  dxloadobject(&obj[2], "models\\tank2.3do");
  dxloadobject(&obj[3], "models\\turret2.3do");
  dxloadobject(&obj[4], "models\\stank.3do");
  dxloadobject(&obj[5], "models\\sturret.3do");
  dxloadobject(&obj[6], "models\\stank2.3do");
  dxloadobject(&obj[7], "models\\sturret2.3do");
  dxloadobject(&obj[8], "models\\mars.3do");
  dxloadobject(&obj[9], "models\\mturret.3do");
  dxloadobject(&obj[10], "models\\mars2.3do");
  dxloadobject(&obj[11], "models\\mturret2.3do");
  dxloadobject(&obj[12], "models\\humvee.3do");
  dxloadobject(&obj[13], "models\\hturret.3do");
  dxloadobject(&obj[14], "models\\humvee2.3do");
  dxloadobject(&obj[15], "models\\hturret2.3do");
  dxloadobject(&obj[16], "models\\pritsche.3do");
  dxloadobject(&obj[17], "models\\nothing.3do");
  dxloadobject(&obj[18], "models\\prit2.3do");
  dxloadobject(&obj[19], "models\\nothing.3do");
  dxloadobject(&obj[20], "models\\tonner.3do");
  dxloadobject(&obj[21], "models\\nothing.3do");
  dxloadobject(&obj[22], "models\\tonner2.3do");
  dxloadobject(&obj[23], "models\\nothing.3do");
  dxloadobject(&obj[24], "models\\bus.3do");
  dxloadobject(&obj[25], "models\\nothing.3do");
  dxloadobject(&obj[26], "models\\bus2.3do");
  dxloadobject(&obj[27], "models\\nothing.3do");
  dxloadobject(&obj[28], "models\\wolf.3do");
  dxloadobject(&obj[29], "models\\nothing.3do");
  dxloadobject(&obj[30], "models\\wolf2.3do");
  dxloadobject(&obj[31], "models\\nothing.3do");
  dxloadobject(&obj[32], "models\\house.3do");
  dxloadobject(&obj[33], "models\\nothing.3do");
  dxloadobject(&obj[34], "models\\house.3do");  
  dxloadobject(&obj[35], "models\\nothing.3do");
  dxloadobject(&obj[36], "models\\house2.3do");
  dxloadobject(&obj[37], "models\\nothing.3do");
  dxloadobject(&obj[38], "models\\house2.3do");
  dxloadobject(&obj[39], "models\\nothing.3do");
  dxloadobject(&obj[40], "models\\golf.3do");// 10
  dxloadobject(&obj[41], "models\\nothing.3do");
  dxloadobject(&obj[42], "models\\golf.3do");
  dxloadobject(&obj[43], "models\\nothing.3do");
  dxloadobject(&obj[44], "models\\golf2.3do");//11
  dxloadobject(&obj[45], "models\\nothing.3do");
  dxloadobject(&obj[46], "models\\golf2.3do");
  dxloadobject(&obj[47], "models\\nothing.3do");
/*  dxloadobject(&obj[48], "models\\tank.3do");//12 buffer
  dxloadobject(&obj[49], "models\\tank.3do");
  dxloadobject(&obj[50], "models\\tank.3do");
  dxloadobject(&obj[51], "models\\tank.3do");*/
  status(323);
//  mesh.Load_XFile("cube.x", gs->d3d->d3ddevice);
  hwstatus("Procedure dxinit() done...\n");
}

void renderobj(dx3dobject* obj, float x, float y, float z, float a, float b, float c, bool dead){
  int i;
  //  D3DBVERTEX v[3];
  // transformations
  matrix=makematrix(tcos(a),tsin(a),0,0, -tsin(a),tcos(a),0,0, 0,0,1,0, x, y, z, 1);
  gs->d3d->d3ddevice->SetTransform(D3DTS_WORLD, &matrix);
  matrix=makematrix(tcos(c),0,-tsin(c),0, 0,1,0,0, tsin(c),0,tcos(c),0, 0, 0, 0, 1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
  matrix=makematrix(1,0,0,0, 0,tcos(b),-tsin(b),0, 0,tsin(b),tcos(b),0, 0, 0, 0, 1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
  // drawing
  if(obj->lists>0){
    for(i=0;i<obj->lists;i++){
//      gs->d3d->d3ddevice->SetRenderState(D3DRS_TEXTUREHANDLE, dxhtexture[obj.list[i].tex]);
      if(dead){
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[10]);
      } else {
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[obj->list[i].tex]);
      }
//      gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLELIST,D3DBFVF_VERTEX,&obj->list[i].v[0],obj->list[i].num,0);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLELIST, obj->list[i].num, &obj->list[i].v[0] ,sizeof(D3DBVERTEX));
    }
  }
  if(obj->strips>0){
    for(i=0;i<obj->strips;i++){
//      gs->d3d->d3ddevice->SetRenderState(D3DRS_TEXTUREHANDLE, dxhtexture[obj.strip[i].tex]);
      if(dead){
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[10]);
      } else {
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[obj->strip[i].tex]);
      }
//      gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_VERTEX,&obj->strip[i].v[0],obj->strip[i].num,0);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, obj->strip[i].num, &obj->strip[i].v[0] ,sizeof(D3DBVERTEX));
    }
  }
  //mesh.Draw_XFile(gs->d3d->d3ddevice);
}

void rendersecobj(dx3dobject* obj, float x, float y, float z, float a, float b, float c, float pa, float pb, float pc, float tdp, bool dead){
  int i;
  //  D3DBVERTEX v[3];
  // transformations
  matrix=makematrix(tcos(pa),tsin(pa),0,0, -tsin(pa),tcos(pa),0,0, 0,0,1,0, x, y, z, 1);
  gs->d3d->d3ddevice->SetTransform(D3DTS_WORLD, &matrix);
  matrix=makematrix(tcos(pc),0,-tsin(pc),0, 0,1,0,0, tsin(pc),0,tcos(pc),0, 0, 0, 0, 1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
  matrix=makematrix(1,0,0,0, 0,tcos(pb),-tsin(pb),0, 0,tsin(pb),tcos(pb),0, 0, 0, 0, 1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
  matrix=makematrix(tcos(a),tsin(a),0,0, -tsin(a),tcos(a),0,0, 0,0,1,0, 0, 0, 0, 1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
  matrix=makematrix(tcos(c),0,-tsin(c),0, 0,1,0,0, tsin(c),0,tcos(c),0, 0, 0, 0, 1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
  matrix=makematrix(1,0,0,0, 0,tcos(b),-tsin(b),0, 0,tsin(b),tcos(b),0, 0, 0, 0, 1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
  matrix=makematrix(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,-tdp,0,1);
  gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
  // drawing
  if(obj->lists>0){
    for(i=0;i<obj->lists;i++){
//      gs->d3d->d3ddevice->SetRenderState(D3DRS_TEXTUREHANDLE, dxhtexture[obj.list[i].tex]);
      if(dead){
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[10]);
      } else {
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[obj->list[i].tex]);
      }
//      gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLELIST,D3DBFVF_VERTEX,&obj->list[i].v[0],obj->list[i].num,0);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLELIST, obj->list[i].num, &obj->list[i].v[0] ,sizeof(D3DBVERTEX));
    }
  }
  if(obj->strips>0){
    for(i=0;i<obj->strips;i++){
//      gs->d3d->d3ddevice->SetRenderState(D3DRS_TEXTUREHANDLE, dxhtexture[obj.strip[i].tex]);
      if(dead){
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[10]);
      } else {
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[obj->strip[i].tex]);
      }
//      gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_VERTEX,&obj->strip[i].v[0],obj->strip[i].num,0);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, obj->strip[i].num, &obj->strip[i].v[0] ,sizeof(D3DBVERTEX));
    }
  }
}


bool isvisible(float x, float y, float z, float space)
{
	D3DVECTOR mv=makevector(x, y, z);
	mapply2d(&mv);
	return (mv.x > -(1.0f+space) && mv.x < (1.0f+space) && mv.y > -(1.0f+space) && mv.y < (1.0f+space));
}

void rendertrees(bool colored){
  D3DBVERTEX vx[4];
  D3DVECTOR v[4];
  gs->d3d->d3ddevice->SetTextureStageState( 0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
  gs->d3d->d3ddevice->SetTextureStageState( 0, D3DTSS_ADDRESSV, D3DTADDRESS_CLAMP);
  if(colored){
    gs->d3d->d3ddevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);//SRCCOLOR
    gs->d3d->d3ddevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);//INVSRCCOLOR
  } else {
    gs->d3d->d3ddevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ZERO);//SRCCOLOR
    gs->d3d->d3ddevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);//INVSRCCOLOR
  }
  //tree
  int ftopend=TOPEND;
  if(view.firstperson)ftopend+=FIRSTTOPPLUS;
  for(int i=0;i<TREES;i++){
    matrix=makematrix(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1);
    gs->d3d->d3ddevice->SetTransform(D3DTS_WORLD, &matrix);
	bool vis = view.firstperson
		?(tree[i].x>view.px+LEFTEND&&tree[i].x<view.px+RIGHTEND&&tree[i].y<view.py+ftopend&&tree[i].y>view.py+BOTTOMEND)
		:isvisible(tree[i].x, tree[i].y, tree[i].z, 0.2f);
    if(tree[i].used&&vis){
      matrix=makematrix(1,0,0,0, 0,1,0,0, 0,0,1,0, tree[i].x,tree[i].y,tree[i].z,1);
      gs->d3d->d3ddevice->SetTransform(D3DTS_WORLD, &matrix);
      matrix=makematrix(tcos(-tree[i].a),-tsin(-tree[i].a),0,0, tsin(-tree[i].a),tcos(-tree[i].a),0,0, 0,0,1,0, 0,0,0,1);
      gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
      matrix=makematrix(1,0,0,0, 0,tcos(tree[i].b),tsin(tree[i].b),0, 0,-tsin(tree[i].b),tcos(tree[i].b),0, 0,0,0,1);
      gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
      matrix=makematrix(tcos(tree[i].a),-tsin(tree[i].a),0,0, tsin(tree[i].a),tcos(tree[i].a),0,0, 0,0,1,0, 0,0,0,1);
      gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
      gs->d3d->d3ddevice->SetTexture(0, dxstexture[TREESPACEOFF+tree[i].typ]);
      v[0]=makevector(-1, 0.5f, -5);
      v[1]=makevector(1, -0.5f, -5);
      v[2]=makevector(-1, 0.5f, 0);
      v[3]=makevector(1, -0.5f, 0);
      vx[0]=makevertex(v[0], DXNORM(v[0], v[1], v[2]), 0, 0);
      vx[1]=makevertex(v[1], DXNORM(v[0], v[1], v[2]), 1, 0);
      vx[2]=makevertex(v[2], DXNORM(v[0], v[1], v[2]), 0, 1);
      vx[3]=makevertex(v[3], DXNORM(v[0], v[1], v[2]), 1, 1);
//      gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_VERTEX,&vx[0],4,0);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vx[0],sizeof(D3DBVERTEX));
      v[0]=makevector(-1,-0.5f, -5);
      v[1]=makevector( 1, 0.5f, -5);
      v[2]=makevector(-1,-0.5f, 0);
      v[3]=makevector( 1, 0.5f, 0);
      vx[0]=makevertex(v[0], DXNORM(v[0], v[1], v[2]), 0, 0);
      vx[1]=makevertex(v[1], DXNORM(v[0], v[1], v[2]), 1, 0);
      vx[2]=makevertex(v[2], DXNORM(v[0], v[1], v[2]), 0, 1);
      vx[3]=makevertex(v[3], DXNORM(v[0], v[1], v[2]), 1, 1);
//      gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_VERTEX,&vx[0],4,0);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vx[0],sizeof(D3DBVERTEX));
      v[0]=makevector(-0.5f,-0.7f, -5);
      v[1]=makevector( 0.5f, 0.7f, -5);
      v[2]=makevector(-0.5f,-0.7f, 0);
      v[3]=makevector( 0.5f, 0.7f, 0);
      vx[0]=makevertex(v[0], DXNORM(v[0], v[1], v[2]), 0, 0);
      vx[1]=makevertex(v[1], DXNORM(v[0], v[1], v[2]), 1, 0);
      vx[2]=makevertex(v[2], DXNORM(v[0], v[1], v[2]), 0, 1);
      vx[3]=makevertex(v[3], DXNORM(v[0], v[1], v[2]), 1, 1);
//      gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_VERTEX,&vx[0],4,0);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vx[0],sizeof(D3DBVERTEX));
      v[0]=makevector(-0.5f, 0.7f, -5);
      v[1]=makevector( 0.5f,-0.7f, -5);
      v[2]=makevector(-0.5f, 0.7f, 0);
      v[3]=makevector( 0.5f,-0.7f, 0);
      vx[0]=makevertex(v[0], DXNORM(v[0], v[1], v[2]), 0, 0);
      vx[1]=makevertex(v[1], DXNORM(v[0], v[1], v[2]), 1, 0);
      vx[2]=makevertex(v[2], DXNORM(v[0], v[1], v[2]), 0, 1);
      vx[3]=makevertex(v[3], DXNORM(v[0], v[1], v[2]), 1, 1);
//      gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_VERTEX,&vx[0],4,0);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vx[0],sizeof(D3DBVERTEX));
    }
  }
  gs->d3d->d3ddevice->SetTextureStageState( 0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
  gs->d3d->d3ddevice->SetTextureStageState( 0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);
}

void rendershot(int n){
  D3DBVERTEX vx[4];
  matrix=makematrix(tcos(speedgs.g),tsin(speedgs.g),0,0, -tsin(speedgs.g),tcos(speedgs.g),0,0, 0,0,1,0, shot[n].x, shot[n].y, shot[n].z, 1);
  gs->d3d->d3ddevice->SetTransform(D3DTS_WORLD, &matrix);
  if(shot[n].typ==2){
    gs->d3d->d3ddevice->SetTexture(0, NULL);
  } else{
    gs->d3d->d3ddevice->SetTexture(0, dxstexture[0]);
  }
  float size = 0.6f;
//  if(shot[n].typ==0)size=1;
  if(shot[n].typ>=1)size=0.6f;
  vx[0]=makevertex(makevector(-0.08f*size, -0.08f*size, 0), makevector(0,0,-1), 0, 0);
  vx[1]=makevertex(makevector(-0.08f*size, 0.08f*size, 0), makevector(0,0,-1), 0, 1);
  vx[2]=makevertex(makevector(0.08f*size, -0.08f*size, 0), makevector(0,0,-1), 1, 0);
  vx[3]=makevertex(makevector(0.08f*size, 0.08f*size, 0), makevector(0,0,-1), 1, 1);
//  gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_VERTEX,&vx[0],4,0);
  gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vx[0],sizeof(D3DBVERTEX));
}

void renderparticles(bool colored){
  if(!colored){
    gs->d3d->d3ddevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ZERO);//SRCCOLOR
    gs->d3d->d3ddevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);//INVSRCCOLOR
  }
  int ftopend=TOPEND;
  if(view.firstperson)ftopend+=FIRSTTOPPLUS;
  for(int n=0;n<PARTICLES;n++){
    matrix=makematrix(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1);
    gs->d3d->d3ddevice->SetTransform(D3DTS_WORLD, &matrix);
	bool vis = view.firstperson?
	  part[n].x>view.px+LEFTEND&&part[n].x<view.px+RIGHTEND&&part[n].y<view.py+ftopend&&part[n].y>view.py+BOTTOMEND-5
	  :isvisible(part[n].x, part[n].y, part[n].z, 0.25f);
	if(part[n].used&&vis){
      matrix=makematrix(1,0,0,0, 0,1,0,0, 0,0,1,0, part[n].x, part[n].y, part[n].z, 1);
      gs->d3d->d3ddevice->SetTransform(D3DTS_WORLD, &matrix);
      if(view.firstperson){
        matrix=makematrix(1,0,0,0, 0,tcos(-0.5),tsin(-0.5),0, 0,-tsin(-0.5),tcos(-0.5),0, 0,0,0,1);
        gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
	  }
      D3DBLVERTEX vx[4];
      unsigned char opy;
      if(speedgs.g-part[n].t<PARTICLELIFE*EXPLFIRE&&part[n].typ==1){
        if(colored){
          gs->d3d->d3ddevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);//SRCCOLOR
          gs->d3d->d3ddevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);//ONE
        }
        opy=(unsigned char)((1.0-(speedgs.g-part[n].t)/(PARTICLELIFE*EXPLFIRE))*255.0);
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[3]);
        vx[0]=makelvertex(makevector(-0.65f*part[n].size, -0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 0, 0);
        vx[1]=makelvertex(makevector(-0.65f*part[n].size, 0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 0, 1);
        vx[2]=makelvertex(makevector(0.65f*part[n].size, -0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 1, 0);
        vx[3]=makelvertex(makevector(0.65f*part[n].size, 0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 1, 1);
//        gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_LVERTEX,&vx[0],4,0);
        gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vx[0],sizeof(D3DBLVERTEX));
      }
      if(speedgs.g-part[n].t<PARTICLELIFE4*EXPLFIRE&&part[n].typ==4){
        if(colored){
          gs->d3d->d3ddevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);//SRCCOLOR
          gs->d3d->d3ddevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);//ONE
        }
        opy=(unsigned char)((1.0-(speedgs.g-part[n].t)/(PARTICLELIFE4*EXPLFIRE))*255.0);
        gs->d3d->d3ddevice->SetTexture(0, dxstexture[3]);
        vx[0]=makelvertex(makevector(-0.65f*part[n].size, -0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 0, 0);
        vx[1]=makelvertex(makevector(-0.65f*part[n].size, 0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 0, 1);
        vx[2]=makelvertex(makevector(0.65f*part[n].size, -0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 1, 0);
        vx[3]=makelvertex(makevector(0.65f*part[n].size, 0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 1, 1);
//        gs->d3d->d3ddevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DBFVF_LVERTEX,&vx[0],4,0);
        gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vx[0],sizeof(D3DBLVERTEX));
      }
      if(colored){
        gs->d3d->d3ddevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCCOLOR);//SRCCOLOR
        gs->d3d->d3ddevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCCOLOR);//INVSRCCOLOR
      }
      if(part[n].typ==1){
        if(speedgs.g-part[n].t<PARTICLELIFE*EXPLFULLSMOKEBEGIN){
          opy=(unsigned char)(((speedgs.g-part[n].t)/(PARTICLELIFE*EXPLFULLSMOKEBEGIN))*255.0f);
        } else {
          if(speedgs.g-part[n].t<PARTICLELIFE*EXPLFULLSMOKEEND){
            opy=255;
          } else {
            opy=(unsigned char)((1-(speedgs.g-part[n].t-PARTICLELIFE*EXPLFULLSMOKEEND)/(PARTICLELIFE*(1-EXPLFULLSMOKEEND)))*255.0);
          }
        }
      }
      if(part[n].typ==4){
        if(speedgs.g-part[n].t<PARTICLELIFE4*EXPLFULLSMOKEBEGIN){
          opy=(unsigned char)(((speedgs.g-part[n].t)/(PARTICLELIFE4*EXPLFULLSMOKEBEGIN))*255.0f);
        } else {
          if(speedgs.g-part[n].t<PARTICLELIFE4*EXPLFULLSMOKEEND){
            opy=255;
          } else {
            opy=(unsigned char)((1-(speedgs.g-part[n].t-PARTICLELIFE4*EXPLFULLSMOKEEND)/(PARTICLELIFE4*(1-EXPLFULLSMOKEEND)))*255.0);
          }
        }
      }
      if(part[n].typ==3){
        // Rotate
        matrix=makematrix(tcos(part[n].a),tsin(part[n].a),0,0, -tsin(part[n].a),tcos(part[n].a),0,0, 0,0,1,0, 0,0,0,1);
        gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
        float opyf=(1-(speedgs.g-part[n].t)/PARTICLELIFE3)*255.0f;
        if(opyf>255.0f)opyf=255.0f;
        if(opyf<0.0)opyf=0.0;
        opy=(unsigned char)opyf;
      }
      if(part[n].typ==5){
        // Rotate
        matrix=makematrix(tcos(part[n].a),tsin(part[n].a),0,0, -tsin(part[n].a),tcos(part[n].a),0,0, 0,0,1,0, 0,0,0,1);
        gs->d3d->d3ddevice->MultiplyTransform(D3DTS_WORLD, &matrix);
        float opyf=(1-(speedgs.g-part[n].t)/PARTICLELIFE5)*255.0f;
        if(opyf>255.0f)opyf=255.0f;
        if(opyf<0.0)opyf=0.0;
        opy=(unsigned char)opyf;
      }
      gs->d3d->d3ddevice->SetTexture(0, dxstexture[4]);
      vx[0]=makelvertex(makevector(-0.65f*part[n].size, -0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 0, 0);
      vx[1]=makelvertex(makevector(-0.65f*part[n].size, 0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 0, 1);
      vx[2]=makelvertex(makevector(0.65f*part[n].size, -0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 1, 0);
      vx[3]=makelvertex(makevector(0.65f*part[n].size, 0.65f*part[n].size, 0), RGBA_MAKE(opy,opy,opy,opy), 1, 1);
      gs->d3d->d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,&vx[0],sizeof(D3DBLVERTEX));
    }
  }
}

inline void removelight(int n){
  part[n].lit=false;
}

inline void killparticle(int n){
  removelight(n);
  part[n].used=false;
}

inline void killshot(int n){
  shot[n].used=false;
#ifndef DISABLESHOTLIGHTING
  gs->d3d->d3ddevice->LightEnable(SHOTLIGHTOFFSET+n, FALSE);
  gs->d3d->d3ddevice->SetLight(SHOTLIGHTOFFSET+n, NULL);
  RELEASE(shot[n].light);
#endif
}

void killgame(void){
	int i;
    for(i=0;i<OMILS;i++){
      omil[i].used=false;
    }
    for(i=0;i<NUMSHOTS;i++){
      if(shot[i].used)killshot(i);
      shot[i].used=false;
    }
    for(i=0;i<PARTICLES;i++){
      if(part[i].used)killparticle(i);
      part[i].used=false;
    }
}

void treeinit(void){
  for(int i=0;i<TREES;i++){
	switch(rand()%5)
	{
	case 0:
	    tree[i].x=(float)begr(rand(), FIELDX*TERRAFAC);
		tree[i].y=(float)begr(rand(), FIELDY*TERRAFAC);
		break;
	case 1:
	case 2:
	case 3:
	    tree[i].x=(float)begr(rand(), 288);
		tree[i].y=(float)begr(rand(), 288);
		break;
	case 4:
	    tree[i].x=(float)begr(rand(), 64)-32+144;
		tree[i].y=(float)begr(rand(), 64)-32+144;
		break;
	}
    tree[i].a=0;
    tree[i].b=0;
    tree[i].z=height(tree[i].x, tree[i].y);
    tree[i].typ=begr(rand(), 4);
    tree[i].used=true;
    tree[i].falling=false;
  }
}

void initintrotext(){
	D3DLOCKED_RECT lrect;
  for(int k=0;k<2;k++){
    statusreal(2300);
    statusreal(2301);
    if(dxstexture[TEXTSPACEOFF+k]->LockRect(0, &lrect, NULL, 0)!=D3D_OK)dxerror();
    statusreal(2302);
    unsigned int *texbuf=(unsigned int*)lrect.pBits;
    unsigned short *texbufw=(unsigned short*)lrect.pBits;
    if(lrect.pBits==NULL){statusreal(708);dxerror();}
    statusreal(2303);
    for(int i=0;i<256;i++){
      for(int j=0;j<256;j++){
          texbuf[256*j+i]=TEXTOPACITY<<24;
      }
    }
    statusreal(2304);
    if(dxstexture[TEXTSPACEOFF+k]->UnlockRect(0)!=D3D_OK)dxerror();
    statusreal(2305);
    // Text
    char str[STRINGLEN];
    strcpy(str, "war is hell, but the sound rocks!");
    statusreal(2306);
    T3TextOut(false, &fontsmall[0], TEXTSPACEOFF+k, 60-256*k, 100, str, strlen(str));
    statusreal(2307);
  }
}

void initintrotext2(){
	D3DLOCKED_RECT lrect;
  for(int k=0;k<2;k++){
    statusreal(2300);
    statusreal(2301);
    if(dxstexture[TEXTSPACEOFF+k]->LockRect(0, &lrect, NULL, 0)!=D3D_OK)dxerror();
    statusreal(2302);
    unsigned int *texbuf=(unsigned int*)lrect.pBits;
    unsigned short *texbufw=(unsigned short*)lrect.pBits;
    if(lrect.pBits==NULL){statusreal(708);dxerror();}
    statusreal(2303);
    for(int i=0;i<256;i++){
      for(int j=0;j<256;j++){
          texbuf[256*j+i]=TEXTOPACITY<<24;
      }
    }
    statusreal(2304);
    if(dxstexture[TEXTSPACEOFF+k]->UnlockRect(0)!=D3D_OK)dxerror();
    statusreal(2305);
    // Text
    char str[STRINGLEN];
    strcpy(str, "www.infinitecode.com presents:");
    statusreal(2306);
    T3TextOut(false, &fontsmall[0], TEXTSPACEOFF+k, 80-256*k, 50, str, strlen(str));
    strcpy(str, "Tank 3");
    statusreal(2306);
    T3TextOut(true, &fontbig[0], TEXTSPACEOFF+k, 190-256*k, 150, str, strlen(str));
    statusreal(2307);
  }
}

void gameinit(void){
  int i;
  bool firstmissioninit = false;
  speedgs.g=0;
  view.lastesc = 0;
  stats.uservis[0]=0;
  stats.uservis[1]=1;
  speedgs.frames=0;
  view.changed = true;
  view.changedr = false;
  if(strstr(mission.fname, ".int")!=NULL){
    view.firstperson=true;
    view.introsection=0;
	view.zoom = STDFPZOOM;
  } else {
	if(view.firstperson) firstmissioninit = true;
    view.firstperson=false;
	view.zoom = STDZOOM;
  }
  bool fromfile = false;
  if(strstr(mission.fname, "save") == mission.fname)
  {
	fromfile = true;
  }
//  gs->d3d->d3ddevice->SetRenderState(D3DRS_TEXTUREPERSPECTIVE,TRUE);
  if(view.firstperson){
    initintrotext();
  } else {
//    gs->d3d->d3ddevice->SetRenderState(D3DRS_FOGENABLE,FALSE);
  }
  for(i=0;i<SOUNDS;i++)lastplayed[i]=0;
  for(i=0;i<NUMSHOTS;i++)if(shot[i].used)killshot(i);
  for(i=0;i<OMILS;i++){
    omil[i].used=false;
    omil[i].dead=false;
  }
  for(i=0;i<TREES;i++)tree[i].used=false;
  for(i=0;i<PARTICLES;i++)if(part[i].used)killparticle(i);
  for(i=0;i<TERRAWAYS;i++)terra.way[i].used=false;
  for(i=0;i<TERRAPLAINS;i++)terra.plain[i].used=false;
  mission.success=false;
  mission.done=false;
//  for(i=0;i<PARTICLES;i++)gs->d3d->d3ddevice->LightEnable(i+PARTLIGHTOFFSET, FALSE);
  statusreal(1300);
  loadmission(mission.fname);
  srand(mission.numomil+mission.namelen+3);
  if(!fromfile)
	  mission.snow=(begr(abs(rand()), 2)==0);
  initgreen(); // terraintextur
  dxinittex();
  statusreal(1301);
  // lighting
  dxinitlight();
  statusreal(1302);
//  dxinitlight();
  terrainit(); // heightmap mit ways und plains, draw ways onto texture
  statusreal(1302);
  terranorm();
  statusreal(1304);
  if(!fromfile)
  {
	  for(i=0;i<min(mission.numomil, OMILS);i++){
		omil[i].noorders=false;
		omil[i].nogoal=false;
		omil[i].a=0;
		omil[i].ta=0;
		omil[i].owner=mission.omildesc[i].owner;
		omil[i].lastfoot=speedgs.g;
		omil[i].lastshot=speedgs.g;
		omil[i].lastsmoke=speedgs.g;
		omil[i].typ=mission.omildesc[i].typ;
		omil[i].x=mission.omildesc[i].posx;
		omil[i].y=mission.omildesc[i].posy;
		omil[i].a=mission.omildesc[i].a;
		omil[i].target=mission.omildesc[i].target;
		omil[i].ttarget=-1;
		if(omil[i].target>=0){
		  omil[i].mtarget=true;
		  omil[i].hitting=false;
		} else {
		  omil[i].mtarget=false;
		}
		omil[i].targetx=mission.omildesc[i].targetx;
		omil[i].targety=mission.omildesc[i].targety;
		omil[i].noorders=mission.omildesc[i].noorders;
		omil[i].nogoal=mission.omildesc[i].nogoal;
		omil[i].mustsurvive=mission.omildesc[i].mustsurvive;
		omil[i].mustdie=mission.omildesc[i].mustdie;
		omil[i].thisisstatic=mission.omildesc[i].thisisstatic;
		omil[i].tb=0;
		omil[i].hit=mildsc[omil[i].typ].hitpoints;
		omil[i].maxhit=omil[i].hit;
		omil[i].tdp=0;
		omil[i].used=true;
		omil[i].selected=false;
		// z-position
		omil[i].z=height(omil[i].x, omil[i].y)-0.35f;
		// b, c
		omil[i].b=(float)atan((height(omil[i].x+0.88f*tsin(omil[i].a), omil[i].y-0.88f*tcos(omil[i].a))-height(omil[i].x-0.88f*tsin(omil[i].a), omil[i].y+0.88f*tcos(omil[i].a)))*0.568f);
		omil[i].c=(float)atan((height(omil[i].x-0.66f*tcos(omil[i].a), omil[i].y-0.66f*tsin(omil[i].a))-height(omil[i].x+0.66f*tcos(omil[i].a), omil[i].y+0.66f*tsin(omil[i].a)))*0.757f);
	  }
  }
  statusreal(1305);
  view.menutype=0;
  statusreal(1306);
  view.menutype=-1;
  if(firstmissioninit)
	  displaymenu(6);
  else if(!fromfile)
	  displaymenu(0);
  else
	  view.menu = false;
  if(view.firstperson)view.menu=false;
  statusreal(1307);
  if(!fromfile)
	treeinit();
  statusreal(1308);
  if(view.firstperson)playone(0);
  // time
  speedgs.c=1/20.0f;
  speedgs.lc=1/20.0f;
  struct timeb t2;
  ftime(&t2);
  float f2=0.001f*(1000*t2.time+t2.millitm);
  speedgs.t=f2;
  statusreal(1309);
}

BOOL wininit(HINSTANCE hThisInstance){
  strcpy(szClassName, "Tank 3 Window");
  return true;
}

LRESULT CALLBACK WindowProcedure2(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  POINT p;
  GetCursorPos(&p);
  mouse.x=p.x;
  mouse.y=p.y;
  switch (message){
           case WM_DESTROY:
             PostQuitMessage(0);
           break;
           case WM_KEYDOWN:
             if(wParam==0x51||wParam==0x71)PostQuitMessage(0);
           break;
           default:;
  }
  if(initcomplete){
    switch (message){
           case WM_RBUTTONDOWN:
             mouse.rdown=true;
           break;
           case WM_LBUTTONDOWN:
             if(!mouse.down){
               float x=((2.0f*p.x)/XRES-1.0f),y=(1.0f-(2.0f*p.y)/YRES);
               mouse.downx=x;
               mouse.downy=y;
               mouse.down=true;
             }
           break;
           case WM_LBUTTONUP:
             mouse.up=true;
           break;
           case WM_MOUSEMOVE:
             if(((wParam&MK_LBUTTON)==MK_LBUTTON)&&!mouse.down){
               float x=((2.0f*p.x)/XRES-1.0f),y=(1.0f-(2.0f*p.y)/YRES);
               mouse.downx=x;
               mouse.downy=y;
               mouse.down=true;
             }
             if(((wParam&MK_RBUTTON)==MK_RBUTTON)&&!mouse.rdown){
               mouse.rdown=true;
             }
             if(((wParam&MK_LBUTTON)!=MK_LBUTTON)&&mouse.down&&!mouse.up){
               mouse.up=true;
             }
           break;
           default:;
    }
  }
  return DefWindowProc(hwnd, message, wParam, lParam);
}

inline D3DLIGHT8 partlight(int i){
  D3DLIGHT8 pdxlight;
  float intensity;
  if(speedgs.g-part[i].t<PARTICLELIFE*EXPLFIRE){
    intensity=1.0f-(speedgs.g-part[i].t)/(PARTICLELIFE*EXPLFIRE);
  } else {
    intensity=0;
  }
  memset(&pdxlight, 0, sizeof(D3DLIGHT8));
  pdxlight.Type = D3DLIGHT_POINT;
  pdxlight.Diffuse.r = intensity;//*part[i].size/EXPSIZE;
  pdxlight.Diffuse.g = intensity;//*part[i].size/EXPSIZE;
  pdxlight.Diffuse.b = 0;
  pdxlight.Diffuse.a = 0;//*part[i].size/EXPSIZE;
  pdxlight.Attenuation0 = 0.0;
  pdxlight.Attenuation1 = 0.0;
  pdxlight.Attenuation2 = 1.5;
  pdxlight.Position.x = part[i].x;
  pdxlight.Position.y = part[i].y;
  pdxlight.Position.z = part[i].z;
  pdxlight.Direction.x = 0;
  pdxlight.Direction.y = 0;
  pdxlight.Direction.z = 0;
  pdxlight.Range = 8.0f/**intensity*/*part[i].size/EXPSIZE;
  if(intensity==0)pdxlight.Range = 0;
  return pdxlight;
}

inline D3DLIGHT8 shotlight(int i){
  D3DLIGHT8 pdxlight;
  memset(&pdxlight, 0, sizeof(D3DLIGHT8));
  pdxlight.Type = D3DLIGHT_POINT;
  pdxlight.Diffuse.r = 1;
  pdxlight.Diffuse.g = 1;
  pdxlight.Diffuse.b = 1;
  pdxlight.Attenuation0 = 0.0;
  pdxlight.Attenuation1 = 0.0;
  pdxlight.Attenuation2 = 3.0;
  pdxlight.Position.x = shot[i].x;
  pdxlight.Position.y = shot[i].y;
  pdxlight.Position.z = shot[i].z;
  pdxlight.Direction.x = 0;
  pdxlight.Direction.y = 0;
  pdxlight.Direction.z = 0;
  pdxlight.Range = 2.0f;
  return pdxlight;
}

void newparticle(int typ, float x, float y, float z, float size, bool lit) {
  for(int i=0;i<PARTICLES;i++){
    if(!part[i].used){
      part[i].used=true;
      part[i].typ=typ;
      part[i].x=x;
      part[i].y=y;
      part[i].z=z;
      part[i].size=size;
      part[i].t=speedgs.g;
      part[i].dz=-0.25;
      part[i].lit=lit;
      if(lit){
        // lighting
//        gs->d3d->CreateLight(&part[i].light, NULL);
//        for(j=PARTLIGHTOFFSET;j<MAXLIGHTS+PARTLIGHTOFFSET;j++){
//          if(!lightused[j]){
/*            dxlight=partlight(i);
//            part[i].light->SetLight(&dxlight);
            gs->d3d->d3ddevice->SetLight(i+PARTLIGHTOFFSET, &dxlight);
            gs->d3d->d3ddevice->LightEnable(i+PARTLIGHTOFFSET, TRUE);*/
//            part[i].lightid=j;
//            lightused[]=true;
//          }
//        }
      }
      break;
    }
  }
}

void speed(void){
  struct timeb t2;
  ftime(&t2);
  float f2=0.001f*(1000*t2.time+t2.millitm);
  speedgs.c=0.9f*speedgs.c+0.1f*(f2-speedgs.t);
  if(speedgs.c>1.0f/5.0f)speedgs.c=1.0f/5.0f;
  if(speedgs.c<0)speedgs.c=0;
//  speedgs.lc=0.997*speedgs.lc+0.003*(f2-speedgs.t);
  speedgs.lc=0.98f*speedgs.lc+0.02f*(f2-speedgs.t);
  speedgs.g+=speedgs.c;
  speedgs.frames++;
  if(speedgs.lc!=0)stats.uservis[2]=1/speedgs.lc/20.0f;
  speedgs.t=f2;
  // counter
  speedgs.cnt++;
}

inline void addfoot(int n){
  for(int i=-8;i<8;i+=2){
    addpixel((omil[n].x+0.45f*tcos(omil[n].a)+0.1f*i*tsin(omil[n].a))/TERRAFAC, (omil[n].y+0.45f*tsin(omil[n].a)-0.1f*i*tcos(omil[n].a))/TERRAFAC);
    addpixel((omil[n].x-0.45f*tcos(omil[n].a)+0.1f*i*tsin(omil[n].a))/TERRAFAC, (omil[n].y-0.45f*tsin(omil[n].a)-0.1f*i*tcos(omil[n].a))/TERRAFAC);
  }
}

inline void addsmoke(int n){
  newparticle(5, omil[n].x+0.35f*mildsc[omil[n].typ].size*tcos(omil[n].a),
	  omil[n].y+0.35f*mildsc[omil[n].typ].size*tsin(omil[n].a), 
	  height(omil[n].x+0.35f*mildsc[omil[n].typ].size*tcos(omil[n].a), 
	  omil[n].y+0.35f*mildsc[omil[n].typ].size*tsin(omil[n].a)),0.6f, false);
  newparticle(5, omil[n].x-0.35f*mildsc[omil[n].typ].size*tcos(omil[n].a), 
	  omil[n].y-0.35f*mildsc[omil[n].typ].size*tsin(omil[n].a), 
	  height(omil[n].x-0.35f*mildsc[omil[n].typ].size*tcos(omil[n].a), 
	  omil[n].y-0.35f*mildsc[omil[n].typ].size*tsin(omil[n].a)),0.6f, false);
}

inline void einschlag(float x, float y, int typ){
  int i, j;
  for(i=-4;i<=4;i++){
    for(j=-4;j<=4;j++){
      if((i*i+j*j)<=4){
        addpixel((x+(float)i/FXRES)/TERRAFAC, (y+(float)j/FYRES)/TERRAFAC);
      }  
    }
  }
  for(i=-1;i<=1;i++){
    for(j=-1;j<=1;j++){
      if((i*i+j*j)<=1){
        addpixel((x+(float)i/FXRES)/TERRAFAC, (y+(float)j/FYRES)/TERRAFAC);
      }  
    }
  }
  if(typ!=3||begr(rand(), 2)==0)newparticle(3, x, y, height(x, y), 1, false);
  addpixel(x, y);
}

void explode(int n){
  int i;
  float size=EXPSIZE;
  if(omil[n].typ==8||omil[n].typ==9)size*=2;
  newparticle(1, omil[n].x, omil[n].y, omil[n].z-0.5f, size, false);
  if(EXPPARTS>0){
    for(i=0;i<EXPPARTS;i++){
      newparticle(1, omil[n].x+begrf(rand()/1000.0f, 2)-1, omil[n].y+begrf(rand()/1000.0f, 2)-1, omil[n].z-1+begrf(rand()/1000.0f, 2), size*0.8f-begrf(rand()/1000.0f, EXPSIZE*0.6f), false);
    }
  }
  omil[n].used=false;
  omil[n].dead=true;
  for(i=0;i<OMILS;i++){
    if(omil[i].target==n){
      omil[i].target=-1;
      omil[i].mtarget=false;
    }
    if(omil[i].ttarget==n){
      omil[i].ttarget=-1;
      omil[i].mtarget=false;
    }
  }
  for(i=0;i<NUMSHOTS;i++){
    if(shot[i].tank==n)shot[i].tank=-1;
  }
  play(1);
}

void panic(int n){
  float mindst=1E20f;
  int ind=-1;
  for(int i=0;i<OMILS;i++){
    if(omil[i].used&&i!=n&&omil[i].owner!=omil[n].owner&&omil[i].owner!=3){
      float dst=(omil[i].x-omil[n].x)*(omil[i].x-omil[n].x)
               +(omil[i].y-omil[n].y)*(omil[i].y-omil[n].y);
      if(dst<mindst){
        mindst=dst;
        ind=i;
      }
    }
  }
  if(ind!=-1){
    attack(n, ind);
  }
}

inline void checkhit(int n, int s){
  if(absf(omil[n].x-shot[s].x)<mildsc[omil[n].typ].size&&absf(omil[n].y-shot[s].y)<mildsc[omil[n].typ].size&&absf(omil[n].z-shot[s].z)<mildsc[omil[n].typ].size&&(shot[s].owner!=omil[n].owner||shot[s].typ==-1)){
    if(shot[s].tank!=-1){
      omil[shot[s].tank].hitting=true;
    }
    if(omil[n].owner!=0&&omil[n].target==-1&&omil[n].ttarget==-1&&shot[s].tank!=-1){
      panic(n);
    }
/*    if(omil[n].ttarget==-1&&shot[s].tank!=-1)omil[n].ttarget=shot[s].tank;*/
    float hit=mildsc[shot[s].typ==-1?2:shot[s].typ].shothit;
    if(view.firstperson)hit*=10;
    omil[n].hit-=hit;
    killshot(s);
    if((shot[s].typ!=3||begr(rand(), 8)==0)
    &&(shot[s].typ!=-1||begr(rand(), 3)==0))newparticle(4, shot[s].x, shot[s].y, shot[s].z-0.5f, HITSIZE-begrf(rand()/1000.0f, HITSIZERANGE), false);
    if(omil[n].hit<0){
      explode(n);
    }
  }
}

void calcki(){
  for(int i=0;i<OMILS;i++){
    if(omil[i].used&&!omil[i].thisisstatic){
      ki(i);
    }
  }
}

void calcfoot(){
  for(int i=0;i<OMILS;i++){
    if(omil[i].used&&!omil[i].thisisstatic){
      if((omil[i].sp!=0&&omil[i].da!=0)&&speedgs.g>omil[i].lastfoot+0.6/omil[i].sp){
        omil[i].lastfoot=speedgs.g;
        addfoot(i);
      }
      if((omil[i].sp!=0&&omil[i].da!=0)&&speedgs.g>omil[i].lastsmoke+0.6/omil[i].sp){
        omil[i].lastsmoke=speedgs.g;
        addsmoke(i);
      }
    }
  }
}

void calccol(void){
  int i, j;
  float xdst;// distance in x-direction
  float ydst;// distance in y-direction
  float dst, dstsq;// direct distance (calculated with pythagoras)
  // for all vehicles...
  for(i=0;i<OMILS-1;i++){
    // ... that exist, if we are not in the intro, also the dead vehicles
    if(omil[i].used||(omil[i].dead&&!view.firstperson)){
      // compare with all other vehicles that we've not calculated yet ...
      for(j=i+1;j<OMILS;j++){
        // ... that exist, if we are not in the intro, also the dead vehicles
        if(i!=j&&(omil[j].used||(omil[j].dead&&!view.firstperson))){
          xdst=omil[i].x-omil[j].x;// calculate distance in x-direction
          ydst=omil[i].y-omil[j].y;// calc dist in y-dir
          float size;// minimal space between two vehicles
          // add the sizes of the two vehicles to get the space
          size=mildsc[omil[i].typ].size+mildsc[omil[j].typ].size;
          // dead ones are calculated smaller to avoid moving problems with
          // tanks in front of lots of dead vehicles
          if(omil[i].dead||omil[j].dead)size*=0.8f;
          // calculate the distance with pythagoras
          dstsq=xdst*xdst+ydst*ydst;
          // if the distance is lower than size we have a collision
          if(dstsq<size*size){
			dst = (float)sqrt(dstsq);
            if(omil[i].target==j)omil[i].hitting=true;// just for ai
            if(omil[j].target==i)omil[j].hitting=true;// just for ai
            // calculate the moving values to increase the distance between
            // the tanks, each tank does half of the movement
            xdst=0.5f*(xdst/dst)*(size-dst);
            ydst=0.5f*(ydst/dst)*(size-dst);
            // move first tank
            // houses and dead vehicles don't move
            if(omil[i].used&&!omil[i].thisisstatic){
              omil[i].x+=xdst;// move the tank
              omil[i].y+=ydst;
              // calculate the angle of the colliding tank
              float atot=0;
              if((omil[j].y-omil[i].y)!=0)atot=(float)atan((omil[j].x-omil[i].x)/(omil[j].y-omil[i].y));
              if((omil[j].y-omil[i].y)<0)atot+=PI;
              float aot=atot+omil[i].a;
              // keep angle between -PI...PI
              while(aot>PI)aot-=2*PI;
              while(aot<-PI)aot+=2*PI;
              // If the other tank collided with our left side, turn right a bit
              // If the other tank collided with our right side, turn left a bit
              // This prevents tanks that seem to be moving "sidewards"
              if(aot>0&&aot<PI/2)omil[i].a+=speedgs.c*1.3f;
              if(aot<0&&aot>-PI/2)omil[i].a-=speedgs.c*1.3f;
            }
            // now the second tank
            // houses and dead vehicles don't move
            if(omil[j].used&&!omil[j].thisisstatic){
              omil[j].x-=xdst;
              omil[j].y-=ydst;
              // calculate the angle of the colliding tank
              float atot2=0;
              if((omil[i].y-omil[j].y)!=0)atot2=(float)atan((omil[i].x-omil[j].x)/(omil[i].y-omil[j].y));
              if((omil[i].y-omil[j].y)<0)atot2+=PI;
              float aot2=atot2+omil[j].a;
              // keep angle between -PI...PI
              while(aot2>PI)aot2-=2*PI;
              while(aot2<-PI)aot2+=2*PI;
              // If the other tank collided with our left side, turn right a bit
              // If the other tank collided with our right side, turn left a bit
              if(aot2>0&&aot2<PI/2)omil[j].a+=speedgs.c*1.3f;
              if(aot2<0&&aot2>-PI/2)omil[j].a-=speedgs.c*1.3f;
            }
          }
        }
      }
    }
  }
}

void calctreecol(void){
//  float xdst, ydst, dstsq;
  for(int i=0;i<OMILS;i++){
    if(omil[i].used&&!omil[i].thisisstatic){
      for(int j=0;j<TREES;j++){
        if(tree[j].used&&!tree[j].falling){
          if(absf(omil[i].x-tree[j].x)<mildsc[omil[i].typ].size&&absf(omil[i].y-tree[j].y)<mildsc[omil[i].typ].size){
            // Baum fllt tot um
            if(omil[i].sp>=0){
              tree[j].a=omil[i].a;
            } else {
              tree[j].a=omil[i].a+PI;
            }
            tree[j].falling=true;
          }
        }
      }
    }
  }
}

void calcsuccess(void){
  int i;
  int friends=0, foes=0;
  for(i=0;i<OMILS;i++){
    if(omil[i].used){
      if(omil[i].owner==0&&!omil[i].nogoal)friends++;
      if(omil[i].owner==1&&!omil[i].nogoal)foes++;
    }
  }
  bool success=true, fail=false;
  if(mission.goal.killall&&foes!=0)success=false;
  if(mission.goal.kill!=-1&&omil[mission.goal.kill].used)success=false;
  if(mission.goal.location){
    for(i=0;i<OMILS;i++){
      if(omil[i].used&&omil[i].owner==0&&!omil[i].nogoal){
        if(((omil[i].x-mission.goal.gotox)*(omil[i].x-mission.goal.gotox)
           +(omil[i].y-mission.goal.gotoy)*(omil[i].y-mission.goal.gotoy))>=mission.goal.radius*mission.goal.radius){
          success=false;
        }
      }
    }
  }
  if(mission.goal.killlocation){
    for(i=0;i<OMILS;i++){
      if(omil[i].used&&omil[i].owner!=0&&!omil[i].nogoal){
        if(((omil[i].x-mission.goal.killx)*(omil[i].x-mission.goal.killx)
           +(omil[i].y-mission.goal.killy)*(omil[i].y-mission.goal.killy))<mission.goal.killradius*mission.goal.killradius){
          success=false;
        }
      }
    }
  }
  if(mission.goal.ban){
    for(i=0;i<OMILS;i++){
      if(omil[i].used&&omil[i].owner!=0&&!omil[i].nogoal){
        if(((omil[i].x-mission.goal.banx)*(omil[i].x-mission.goal.banx)
           +(omil[i].y-mission.goal.bany)*(omil[i].y-mission.goal.bany))<mission.goal.banradius*mission.goal.banradius){
          success=false;
          fail=true;
        }
      }
    }
  }
  // Must die unit still lives
  for(i=0;i<OMILS;i++){
    if(omil[i].used&&omil[i].mustdie){
      success=false;
    }
  }
  // Must survive unit shot
  for(i=0;i<OMILS;i++){
    if((!omil[i].used)&&omil[i].mustsurvive){
      success=false;
      fail=true;
    }
  }
  if(friends==0){success=false;fail=true;}
  if(mission.goal.survive!=-1){
    if(!omil[mission.goal.survive].used){fail=true;success=false;}
  }
  if(success||fail){
    if(view.firstperson){
      mission.success=true;
      if(strlen(mission.next)!=0){
        strcpy(mission.fname, "missions\\");
        strcat(mission.fname, mission.next);
      }
      displaymenu(2);
      view.timemenu=true;
      view.menukilltime=speedgs.g+8;
      mission.done=true;
    } else {
      if(success){
        mission.success=true;
        if(strlen(mission.next)!=0){
          strcpy(mission.fname, "missions\\");
          strcat(mission.fname, mission.next);
        }
      } else {
        mission.success=false;
      }
      mission.done=true;
      if(strcmp(mission.next, "")==0){
        displaymenu(5);
        view.timemenu=true;
        view.menukilltime=speedgs.g+6;
      } else {
        displaymenu(2);
        view.timemenu=true;
        view.menukilltime=speedgs.g+2;
      }
    }
//    gameinit();
  }
}

void calctrees(void){
  for(int i=0;i<TREES;i++){
    if(tree[i].used&&tree[i].falling){
      tree[i].b+=2*speedgs.c;
      if(tree[i].b>0.5*PI)tree[i].b=0.5*PI;
    }
  }
}

void calcmenu(void){
//  stats.uservis[2]=(speedgs.g-view.menukilltime)*0.01+1;
  view.gamepaused=(view.menu&&view.menutype!=5&&view.menutype!=2);
  if(view.newmenu){
    displaymenu(view.newmenun);
    view.newmenu=false;
  }
  if(view.menu&&view.timemenu&&speedgs.g>view.menukilltime){
    if(view.menutype==2||view.menutype==5){
      view.timemenu=false;
      killgame();
      gameinit();
    }
  }
  if(view.menu&&view.menutype==4){
    displaycredits();
  }
}

void calculate(void){
  int i, j;
  if(view.firstperson&&speedgs.g>=6&&view.introsection==0){
    initintrotext2();
    view.introsection=1;
  }
  status(100);
  if(!view.gamepaused)speed();
  startprofile(13);
  reinittextspace2();// display fps
  stopprofile(13);
  startprofile(14);
  calcmenu();
  stopprofile(14);
  status(101);
  if(!view.gamepaused)calctrees();
  status(102);
  startprofile(8);
  status(103);
  if(!view.gamepaused)calcomil();
  stopprofile(8);
  startprofile(9);
  status(104);
  if(!view.gamepaused)calcki();
  status(105);
  stopprofile(9);
  startprofile(10);
  if(!view.gamepaused)calcfoot();
  stopprofile(10);
  startprofile(0);
  if(!mission.done)calcsuccess();
  if(!view.gamepaused)calccol();
  if(!view.gamepaused)calctreecol();
  status(106);
  stopprofile(0);
  startprofile(1);
  if(!view.gamepaused){
//    tparticle swap;
    for(i=0;i<PARTICLES;i++){
      if(part[i].used){
        float plife;
        if(part[i].typ==1)plife=PARTICLELIFE;
        if(part[i].typ==3)plife=PARTICLELIFE3;
        if(part[i].typ==4)plife=PARTICLELIFE4;
        if(part[i].typ==5)plife=PARTICLELIFE5;
        if(speedgs.g-part[i].t>=plife){
          status(107);
          killparticle(i);
          status(108);
        } else {
          part[i].z+=part[i].dz*speedgs.c;
          part[i].size+=0.4f*speedgs.c;
          part[i].a+=((i&1)==0?1.0f:-1.0f)*0.4f*speedgs.c;
          if(speedgs.g-part[i].t>PARTICLELIFE*EXPLFIRE&&part[i].lit&&part[i].typ==1)removelight(i);
          if(speedgs.g-part[i].t>PARTICLELIFE4*EXPLFIRE&&part[i].lit&&part[i].typ==4)removelight(i);
          if(part[i].lit){
            // lighting
/*            dxlight=partlight(i);
            gs->d3d->d3ddevice->SetLight(PARTLIGHTOFFSET+i, &dxlight);*/
          }
        }
      }
    }
    stopprofile(1);
    startprofile(2);
    status(114);
    for(i=0;i<NUMSHOTS;i++){
      if(shot[i].used){
        shot[i].x+=shot[i].dx*speedgs.c;
        shot[i].y+=shot[i].dy*speedgs.c;
        shot[i].z+=shot[i].dz*speedgs.c;
        if(shot[i].typ==2&&speedgs.g-shot[i].lastsmoke>=0.1){
          newparticle(3, shot[i].x, shot[i].y, shot[i].z, 1, false);
          shot[i].lastsmoke=speedgs.g;
        }
        shot[i].dz+=SHOTGRAVITY*speedgs.c;
        if(speedgs.g-shot[i].t>15){
          if(shot[i].tank!=-1)omil[shot[i].tank].hitting=false;
          killshot(i);
          status(116);
          continue;
        }
        if(height(shot[i].x,shot[i].y)<shot[i].z){
          if(shot[i].tank!=-1)omil[shot[i].tank].hitting=false;
          einschlag(shot[i].x, shot[i].y, shot[i].typ);
          killshot(i);
          continue;
        }
        if(shot[i].split!=-1&&shot[i].split<speedgs.g){
          int num=0;
          for(j=0;j<NUMSHOTS;j++){
            if(!shot[j].used){
              num++;
              shot[j]=shot[i];
              shot[j].typ=-1;
              shot[j].split=-1;
              shot[j].dx+=(begr(rand(), 100)-50)*0.04f;
              shot[j].dy+=(begr(rand(), 100)-50)*0.04f;
              shot[j].dz+=(begr(rand(), 100)-50)*0.04f;
            }
            if(num>=10)break;
          }
          shot[i].used=false;
          continue;
        }
#ifndef DISABLESHOTLIGHTING
        // lighting
        dxlight=shotlight(i);
        gs->d3d->d3ddevice->SetLight(SHOTLIGHTOFFSET+i, &dxlight);
//        gs->d3d->d3ddevice->LightEnable(SHOTLIGHTOFFSET+i, TRUE);
#endif
        for(j=0;j<OMILS;j++){
          if(omil[j].used)checkhit(j, i);//the shot might not exist anymore after this
        }
      }
    }
    stopprofile(2);
  }
  status(126);
}

void fontinit(void){
  t3loadfont(&fontbig[0], BIGX, BIGY, "textures\\fontbig.bmp");
  t3loadfont(&fontsmall[0], SMALLX, SMALLY, "textures\\fontsmal.bmp");
}
