#ifndef TANK31FCT_H
#define TANK31FCT_H

#include <d3d8.h>

#include "t3_d3dbase.h"
#include "constants.h"
#include "omil.h"




////////////////////// mouse data //////////////////////7
// down, up:     stores state-changes of the mousebuttons
// rdown:        same for the right button
// x,y:          Position in screen-coordinates
// downx, downy: Position, where the button was pressed
//               Used for drag-select
struct tmouse {
  bool down, up, rdown;
  int x, y;
  float downx, downy;
};
// structs
// dx objects
/////////////// triangle ////////////////////
//
// num: number of vertices
// v:   vertex array
// tex: texture
struct dxtriangle {
  long num;
  D3DBVERTEX* v;
  unsigned tex;
};

//////////////////// 3d-model ////////////////////////
// tank, house, ...
//
// lists, strips, fans: number of directx structures
// list, strip, fan:    triangle arrays
struct dx3dobject {
  long lists, strips, fans;
  dxtriangle *list, *strip, *fan;
};

// objects

//////////// particle ///////////
// explosions or smoke
//
// x,y,z: position
// dz:    change of z per frame, a is the angle
// t:     the time, when the particle was created
// typ:   type of the particle
// used:  this structure is used or empty
// lit:   lighting enabled for this structure
struct tparticle {
  float x, y, z, dz, a;
  float t, size;
  int typ;
  bool used;
  bool lit;
};


///////////////////// tree ///////////////////////////
// x,y,z:   position
// a,b:     angles (for falling trees)
// typ:     type
// used:    used or empty
// falling: A tank just rolled over this tree, so it falls down
struct ttree {
  float x, y, z, a, b;
  char typ;
  bool used, falling;
};

///////////////////////////// shot /////////////////////////
// used:      used or empty
// x,y,z:     the position (again...)
// dx,dy,dz:  movement vector
// t:         the time, when this shot was created
// split:     the time, when the sot splits (only for MARS, typ==2)
// lastsmoke: time, when the last smoke particle was launched
// typ:       type (same as in "struct mil", but -1 for splitted MARS shots)
// owner:     0:you, 1:enemy, 2:civil but hostile, 3:civil and neutral
// tank:      the tank that is responsible for this shot (the one that shooted)
struct tshot{
  bool used;
  float x, y, z;
  float dx,dy,dz;
  float t, split;
  float lastsmoke;
  int typ;
  int owner, tank;
};

/////////////////////////// view ////////////////////////////
// Used to set the camera position (scrolling), calculate menus and more
//
// px,py:         position of the camera for scrolling
// zoom:          The current zoom (changed by + - keys)
// menu:          true, if a menu is displayed currently
// menutype:      which menu (Main, Short Orders, Credits, ...)
// menuscroll:    scrolling index for the "Select Mission"-Menu
// menumaxscroll: maximal number of lines in a scrolling display
// pixscroll:     For per pixel scrollers like the Credits
// menukilltime:  For "mission accomplished" screens
// mfnames:       Mission names for "Select mission"-menu
// firstperson:   true, if the game is in intro mode
//                Used by render-functions, gameplay-functions, ...
// introsection:  Part of the intro. Used for the text blending in the intro.
struct tview {
  float px, py, zoom;
  bool menu, timemenu, newmenu;
  int newmenun;
  bool lastesc, lasto;
  int menutype, menuscroll, menumaxscroll;
  float pixscroll;
  float menukilltime;
  char mfnames[DISPLAYEDMISSIONS][STRINGLEN];
  bool firstperson, gamepaused;
  int introsection;
  bool changed, changedr;
};

////////////////////// gamespeed constants ////////////////////
// t:      time index of last frame
// g:      time since loading of the mission
// cnt:    number for random calculations
// frames: number of frames rendered
// c:      speed constant (seconds per frame)
//         Every movement is multiplied by this value to achieve the same game
//         speed on all computers
// lc:     like c, but an average over a longer time.
//         Used for displaying the framerate
struct tspeed {
  float t;
  float c, lc;
  float g;
  int cnt, frames;
};

extern tshot shot[NUMSHOTS];
extern ttree tree[TREES];
extern tparticle part[PARTICLES];
extern dx3dobject obj[OBJECTS];
extern tview view;
extern tmouse mouse;
extern tspeed speedgs;
extern IDirect3DTexture8 *dxstexture[TEXSPACEOFF+TEXX*TEXY];

extern HWND hwnd;
extern bool initcomplete;
extern bool dxsoftware;

BOOL wininit(HINSTANCE hThisInstance);
void winerror(void);
void fontinit(void);
void inittables(void);
void dxinit(void);
void mildscinit(void);
void dsoundinit();
void inittexttex(void);
void gameinit(void);
void input(void);
void calculate(void);
void render(void);
void killgame(void);
void dsoundshutdown(void);
void dxcleanup(void);
void mapply2d(D3DVECTOR* vec);
void renderparticles(bool colored);
void renderterra(void);
void rendertrees(bool colored);
void rendermil(mil* milobj);
void rendershot(int n);
void statusreal(long s);
void hwstatus(char* str);
void dxerror(void);
void memerror(void);
D3DVECTOR dxfix(float x, float y, float z);
void newparticle(int typ, float x, float y, float z, float size, bool lit);
void play(int num);
void renderobj(dx3dobject* obj, float x, float y, float z, float a, float b, float c, bool dead);
void rendersecobj(dx3dobject* obj, float x, float y, float z, float a, float b, float c, float pa, float pb, float pc, float tdp, bool dead);

#endif