#ifndef TERRA_H
#define TERRA_H

#include "gs.h"
#include "t3_d3dbase.h"
#include "tank31fct.h"

void renderterrastrip(int i, int j, int di, int dj, int &tt, bool ttexture);
void renderterra(void);
void terranorm(void);
float crosslight(D3DVECTOR a, D3DVECTOR b);
float height(float x, float y);
void terrainit(void);
void pathway(float x1, float y1, float x2, float y2, float size);
inline void addpixel(float x,float y);
void initterratex(void);
void initgreen(void);

// size of the playing field
#define FIELDX 192//128
#define FIELDY 192//128
#define TERRAWAYS 100 // Maximal number of ways that can be used in missions
// Maximal number of plains (under houses) that can be used in missions
#define TERRAPLAINS 100 

///////////// way /////////////
// a structure to store ways from a mission file (.mis) and to precalculate
// ways
//
// x1,x2,y1,y2: start and end positions
// typ:         type
// used:        used or empty
struct tway {
  float x1, y1, x2, y2;
  int typ;
  bool used;
};

//////////// plain ///////////
// a structure to store plains from a mission file (.mis) and to precalculate
// plains
//
// x,y :  position
// dx,dy: size
// used:  used or empty
struct tplain {
  float x, y, dx, dy;
  bool used;
};

/////////// terrain ///////////
// the terrain is precalulated and needs large buffers for the vertices
//
// hmap:        map of the height of the terrain
// vtx:         the precalculated vertices of the terrain
// detail:      the precalculated vertices of the terrain detail texture
// tex:         the texture indices
// ways, pains: number of ways and plains on the terrain surface
// way:         array of ways
// plain:       array of surfaces
struct tterrain {
	unsigned char hmap[FIELDX][FIELDY];
	D3DVECTOR vec[FIELDX][FIELDY][4];
	D3DVECTOR lvl0[FIELDX][FIELDY];
	D3DBLVERTEX vtx[FIELDY][FIELDX][4];
	D3DBLVERTEX detail[FIELDY][FIELDX][4];
	D3DBLVERTEX detaill[FIELDY][FIELDX][6];
	unsigned char tex[FIELDX][FIELDY];
	int ways, plains;
	tway way[TERRAWAYS];
	tplain plain[TERRAPLAINS];
	int pbnx[FIELDY], pbxx[FIELDY], pbny, pbxy;
	IDirect3DVertexBuffer8 *vbterra, *vbdetail;
	// terrain ends
	int mini, maxi, minj, maxj;
};

extern tterrain terra;

#endif
