#include "window.h"
#include "errorwindow.h"

cwindowlist cw_wndlist;

cwindowlist::cwindowlist()
{
	list = NULL;
}

cwindowlist::Add(cwindow *wnd)
{
	cwindowlistentry *newwnd = new cwindowlistentry(wnd, list);
	list = newwnd;
}

cwindowlist::Remove(cwindow *wnd)
{
	if(list)
	{
		if(list->wnd == wnd)
		{
			cwindowlistentry *temp = list;
			list = list->next;
			temp->next = 0;
			delete temp;
		}
		else
		{
			cwindowlistentry *search = list;
			while(search->next->wnd !=wnd && search->next)
				search = search->next;
			if(search->next)
			{
				cwindowlistentry *temp = search->next;
				search->next = search->next->next;
				temp->next = NULL;
				delete temp;
			}
		}
	}
}

cwindow *cwindowlist::Get(HWND hwnd)
{
	cwindowlistentry *search = list;
	while(search)
	{
		if(hwnd == search->wnd->hwnd)
			return search->wnd;
	}
	return NULL;
}

cwindowlist::~cwindowlist()
{
	delete list;
}

cwindowlistentry::cwindowlistentry(cwindow *wnd, cwindowlistentry *next)
{
	this->wnd = wnd;
	this->next = next;
}

cwindowlistentry::~cwindowlistentry()
{
	if(next) delete next;
}

LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
//  GetCursorPos(&p);
	cwindow *wnd = cw_wndlist.Get(hwnd);
	if(wnd)
	{
		wnd->onmessage(message, wParam, lParam);
		switch (message){
			case WM_CREATE:
				break;
			case WM_DESTROY:
				PostQuitMessage(0);
				break;
			case WM_TIMER:
				wnd->ontimer();
				break;
			case WM_ACTIVATE:
				if(wParam)
					wnd->onactivate();
				else
					wnd->ondeactivate();
				break;
			case WM_SIZE:
				wnd->onsize(lParam & 0xFFFF, lParam>>16);
				switch(wParam)
				{
				case SIZEFULLSCREEN:
					wnd->onminimax(SW_SHOWMAXIMIZED);
					//MessageBox(NULL, "t", "c", MB_OK);
					break;
				case SIZENORMAL:
					//wnd->onminimax(SW_SHOWNORMAL);
					//MessageBox(NULL, "t", "c", MB_OK);
					break;
				case SIZEICONIC:
					//MessageBox(NULL, "t", "c", MB_OK);
					break;
				}
			break;
			case WM_MOUSEMOVE:
				wnd->onmousemove(lParam&0xFFFF, lParam>>16, wParam);
				break;
			case WM_RBUTTONDOWN:
				wnd->onrbuttondown(lParam&0xFFFF, lParam>>16);
				break;
			case WM_RBUTTONUP:
				wnd->onrbuttonup(lParam&0xFFFF, lParam>>16);
				break;
			case WM_LBUTTONDOWN:
				wnd->onlbuttondown(lParam&0xFFFF, lParam>>16);
				break;
			case WM_LBUTTONUP:
				wnd->onlbuttonup(lParam&0xFFFF, lParam>>16);
				break;
			default:
				return DefWindowProc(hwnd, message, wParam, lParam);
		}
	}
	return DefWindowProc(hwnd, message, wParam, lParam);
}

cwindow::cwindow(HINSTANCE Instance, int WindowStyle, char *caption, int dx, int dy)
{
	this->WindowStyle = WindowStyle;
	this->Instance = Instance;
	// Create Window
	wincl.hInstance = Instance;
	wincl.lpszClassName = WNDCLASSSTR;
	wincl.lpfnWndProc = WindowProcedure;
	wincl.style = 0;//CS_NOCLOSE; 
	wincl.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wincl.hCursor = LoadCursor(NULL, IDC_ARROW);
	wincl.lpszMenuName = NULL;
	wincl.cbClsExtra = 0;
	wincl.cbWndExtra = 0;
	wincl.hbrBackground = (HBRUSH) GetStockObject(LTGRAY_BRUSH);
	clientdx = dx;
	clientdy = dy;
	if(RegisterClass(&wincl))
	{
		hwnd = CreateWindowEx(
			/*WS_EX_TOPMOST*/0,WNDCLASSSTR, caption,	WS_OVERLAPPEDWINDOW,	0,	0, clientdx, clientdy,
			HWND_DESKTOP, NULL, Instance, NULL);
		ShowWindow(hwnd, WindowStyle);
	}
	else
	{
		cerrorwindow("Couldn't register window class", "Windows Error");
	}
	RECT rect;
	GetClientRect(hwnd, &rect);
	clientdx = rect.right - rect.left;
	clientdy = rect.bottom - rect.top;
	cw_wndlist.Add(this);
}

cwindow::~cwindow()
{
	cw_wndlist.Remove(this);
	DestroyWindow(hwnd);
	UnregisterClass(WNDCLASSSTR, Instance);
}

// minimize/maximize event function
cwindow::onminimax(int WindowStyle)
{
	this->WindowStyle = WindowStyle;
}


cwindow::onmessage(int msg, int wparam, int lparam) {}
cwindow::onactivate() {}
cwindow::ondeactivate() {}
cwindow::onrbuttondown(int x, int y) {}
cwindow::onrbuttonup(int x, int y) {}
cwindow::onlbuttondown(int x, int y) {}
cwindow::onlbuttonup(int x, int y) {}
cwindow::onmousemove(int x, int y, int buttons) {}
cwindow::ontimer() {}

cwindow::onsize(int dx, int dy)
{
	clientdx = dx;
	clientdy = dy;
}

cwindow::showmaximized()
{
	ShowWindow(hwnd, SW_SHOWMAXIMIZED);
}

cwindow::shownormal()
{
	ShowWindow(hwnd, SW_SHOWNORMAL);
	onminimax(SW_SHOWNORMAL);
//	SetWindowPos(hwnd, NULL, 0, 0, 800, 600, SWP_NOMOVE|SWP_NOZORDER);
}
